from enum import Enum

class FlashMode(Enum):
    Undefined                   = 0x0000
    Prohibited                  = 0x0002
    RedEyeReduction             = 0x0004

    FrontCurtainSync            = 0x8010
    SlowSync                    = 0x8011
    RearCurtainSync             = 0x8012
    RedEyeReductionSlowSync     = 0x8013



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader

from ptpip.packet.init_cmd_req import InitCmdReq
from ptpip.packet.init_cmd_ack import InitCmdAck
from ptpip.packet.event_req import EventReq
from ptpip.packet.event_ack import EventAck
from ptpip.packet.init_fail import InitFail
from ptpip.packet.cmd_request import CmdRequest
from ptpip.packet.cmd_response import CmdResponse
from ptpip.packet.start_data import StartDataPacket
from ptpip.packet.data import DataPacket
from ptpip.packet.end_data import EndDataPacket
from ptpip.packet.ping import Ping

class PacketFactory():
    def createPacket(
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        request: Packet = None,
        sessionId = None
    ):
        if data is None:
            return None

        reader = StreamReader(data = data)

        packetType = PacketType(reader.readUint32())
        # print('PacketType: ' + packetType.name);

        if packetType == PacketType.InitCmdReq:
            # print("InitCmdReq")
            return InitCmdReq(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode
            )
        elif packetType == PacketType.InitCmdAck:
            # print("InitCmdAck")
            return InitCmdAck(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                sessionId = sessionId
            )
        elif packetType == PacketType.EventReq:
            # print("EventReq")
            return EventReq(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                sessionId = sessionId
            )
        elif packetType == PacketType.EventAck:
            # print("EventAck")
            return EventAck(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                sessionId = sessionId,
                request = request
            )
        elif packetType == PacketType.InitFail:
            # print("InitFail")
            return InitFail(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode
            )
        elif packetType == PacketType.CmdRequest:
            # print("CmdRequest")
            return CmdRequest(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode
            )
        elif packetType == PacketType.CmdResponse:
            # print("CmdResponse")
            return CmdResponse(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                request = request
            )
        # elif packetType == PacketType.Event:
            # print("Event")
        elif packetType == PacketType.StartData:
            # print("StartDataPacket")
            return StartDataPacket(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                request = request
            )
        elif packetType == PacketType.Data:
            # print("DataPacket")
            return DataPacket(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                request = request
            )
        elif packetType == PacketType.EndData:
            # print("EndDataPacket")
            return EndDataPacket(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode,
                request = request
            )
        elif packetType == PacketType.Ping:
            # print("Ping")
            return Ping(
                data = reader.readRest(),
                transactionId = transactionId,
                dataObject = dataObject,
                dataObjectTransferMode = dataObjectTransferMode
            )
        # elif packetType == PacketType.Pong:
            # print("Pong")
        else:
            print("Unknown packet type: " + str(packetType))

        return None



import struct

from ptpip.constants.property_type import PropertyType

class StreamWriter():
    def __init__(self, data = None):
        super(StreamWriter, self).__init__()

        self.data = b'' if data == None else data

    def writeBytes(self, value):
        self.data += value
        return self

    def writeInt8(self, value):
        self.data += struct.pack('b', value)
        return self

    def writeUint8(self, value):
        self.data += struct.pack('B', value)
        return self

    def writeInt16(self, value):
        self.data += struct.pack('h', value)
        return self

    def writeUint16(self, value):
        self.data += struct.pack('H', value)
        return self

    def writeInt32(self, value):
        self.data += struct.pack('i', value)
        return self

    def writeUint32(self, value):
        self.data += struct.pack('I', value)
        return self

    def writeInt64(self, value):
        self.data += struct.pack('q', value)
        return self

    def writeUint64(self, value):
        self.data += struct.pack('Q', value)
        return self

    def writeString(self, value):
        length = len(value)
        self.writeUint8(length)

        for i in range(0, length):
            self.writeUint16(ord(value[i]))

        self.writeUint16(0)

        return self

    def writeArrayByLength(self, typeLength, values):
        length = len(values)
        self.writeUint32(length)

        for i in range(0, length):
            for j in range(0, typeLength):
                self.writeUint8(values[i * typeLength + j])

        return self

    def writeArray(
        self,
        typeName,
        values,
        lengthTypeName = PropertyType.Uint32.name
    ):
        length = len(values)
        self.writeType(lengthTypeName, length)

        for i in range(0, length):
            self.writeType(typeName, values[i])

        return self

    def writeInt8Array(self, values):
        return self.writeArray(PropertyType.Int8.name, values)
    def writeUint8Array(self, values):
        return self.writeArray(PropertyType.Uint8.name, values)

    def writeInt16Array(self, values):
        return self.writeArray(PropertyType.Int16.name, values)
    def writeUint16Array(self, values):
        return self.writeArray(PropertyType.Uint16.name, values)

    def writeInt32Array(self, values):
        return self.writeArray(PropertyType.Int32.name, values)
    def writeUint32Array(self, values):
        return self.writeArray(PropertyType.Uint32.name, values)

    def writeInt64Array(self, values):
        return self.writeArray(PropertyType.Int64.name, values)
    def writeUint64Array(self, values):
        return self.writeArray(PropertyType.Uint64.name, values)

    def writeType(self, typeName, value):
        if typeName == PropertyType.Int8.name:
            return self.writeInt8(value)
        if typeName == PropertyType.Uint8.name:
            return self.writeUint8(value)

        if typeName == PropertyType.Int16.name:
            return self.writeInt16(value)
        if typeName == PropertyType.Uint16.name:
            return self.writeUint16(value)

        if typeName == PropertyType.Int32.name:
            return self.writeInt32(value)
        if typeName == PropertyType.Uint32.name:
            return self.writeUint32(value)

        if typeName == PropertyType.Int64.name:
            return self.writeInt64(value)
        if typeName == PropertyType.Uint64.name:
            return self.writeUint64(value)

        if typeName == PropertyType.String.name:
            return self.writeString(value)

        if typeName == PropertyType.Int8Array.name:
            return self.writeInt8Array(value)
        if typeName == PropertyType.Uint8Array.name:
            return self.writeUint8Array(value)

        if typeName == PropertyType.Int16Array.name:
            return self.writeInt16Array(value)
        if typeName == PropertyType.Uint16Array.name:
            return self.writeUint16Array(value)

        if typeName == PropertyType.Int32Array.name:
            return self.writeInt32Array(value)
        if typeName == PropertyType.Uint32Array.name:
            return self.writeUint32Array(value)

        if typeName == PropertyType.Int64Array.name:
            return self.writeInt64Array(value)
        if typeName == PropertyType.Uint64Array.name:
            return self.writeUint64Array(value)

        print('Unknown type: ' + str(typeName))
        return self



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet

class EventAck(Packet):
    def __init__(
        self,
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        request: Packet = None,
        sessionId = None
    ):
        super(EventAck, self).__init__(
            PacketType.EventAck,
            data = data,
            transactionId = transactionId,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        self.request = request
        self.sessionId = sessionId

    def __str__(self):
        return 'EventAck: ' + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n"



from enum import Enum

# Indicates the choice of “Shooting, recording, and display
# – Live view screen display setting”
# in the custom menu.
class LiveViewScreenDisplaySetting(Enum):
    AllInvalid                  = 0x00
    InformationDisplayOn        = 0x01
    InformationDisplayOff       = 0x02
    GridDisplay                 = 0x04
    InformationScreenDisplay    = 0x08
    AllValid                    = 0x0F



from enum import Enum

class WarningStatus(Enum):
    SequenceError           = 0x01
    BatteryInsufficient     = 0x02
    Reserved                = 0x04
    LensHardwareError       = 0x08
    iTTLError               = 0x10
    MinimumApertureWarning  = 0x20
    BulbWarning             = 0x40
    ChecksumError           = 0x80



from enum import Enum

class NewExternalSpeedLightMode(Enum):
    Off                                 = 0x00
    iTTLBL                              = 0x01
    iTTL                                = 0x02
    ApertureInterlockingAutomaticFlash  = 0x03
    ExternalAutomaticFlash              = 0x04
    ManualDistancePriority              = 0x05
    Manual                              = 0x06
    MultiFlash                          = 0x07



import asyncio

from ptpip.client import PtpIpClient

from ptpip.report.html_device import HtmlDeviceReportGenerator

class GetDeviceInfoCommand():
    def __init__(self, discover, discoverMore, reportFileName):
        super(GetDeviceInfoCommand, self).__init__()

        self.discover = discover
        self.discoverMore = discoverMore
        self.reportFileName = reportFileName

    async def run(self, client: PtpIpClient):
        device = await client.getDeviceInfo()
        # print('Device: ' + str(device))

        props = []
        for idx, prop in enumerate(device.properties):
            propDesc = await client.getDevicePropDesc(
                prop = prop,
                delay = 0
            )
            props.append(propDesc)
            # print('Prop desc(' + str(idx) + '):' + "\n" + str(propDesc))

        discoveredProps = []
        if self.discover:
            discoveredProps = await client.discoverDevicePropDesc(
                device,
                delay = 0.001 if self.discoverMore else 0.010,
                more = self.discoverMore
            )

        # print(str(discoveredProps))

        html = HtmlDeviceReportGenerator(device, props, discoveredProps) \
            .generate()

        f = open(self.reportFileName, 'w')
        f.write(html)
        f.close()

        print('Report saved ! ' + self.reportFileName)



from enum import Enum

class WBTuneFluorescent(Enum):
    SodiumLampMixedLight            = 0x00 # 0
    CoolWhiteFluorescentLamp        = 0x01 # 1
    WarmWhiteFluorescentLamp        = 0x02 # 2
    WhiteFluorescentLamp            = 0x03 # 3
    DayWhiteFluorescentLamp         = 0x04 # 4
    DaylightFluorescentLamp         = 0x05 # 5
    HighColorTemperatureMercuryLamp = 0x06 # 6



import asyncio

from ptpip.constants.property_type import PropertyType
from ptpip.constants.device.property_type import DevicePropertyType
from ptpip.data_object.device_prop_desc import DevicePropDesc

from ptpip.client import PtpIpClient

class SetDevicePropValueCommand():
    def __init__(self, name, id, propTypeName, propTypeId, value):
        super(SetDevicePropValueCommand, self).__init__()

        self.name = name
        self.id = id
        self.propTypeName = propTypeName
        self.propTypeId = propTypeId
        self.value = value

    async def run(self, client: PtpIpClient):
        prop = self.id if self.id != None else DevicePropertyType[self.name].value

        propType = None
        if self.propTypeId != None:
            propType = PropertyType(self.propTypeId)
        elif self.propTypeName != None:
            propType = PropertyType[self.propTypeName]
        else:
            propDesc = await client.getDevicePropDesc(
                prop = prop
            )

            if not isinstance(propDesc, DevicePropDesc):
                raise(Exception('Error while fetching prop desc: ' + str(propDesc)))

            propType = propDesc.propType

        if not isinstance(propType, PropertyType):
            raise(Exception('Property type not found'))

        value = self.value
        if propType.name[0:3] == 'Int' \
            or propType.name[0:4] == 'Uint' \
        :
            value = int(self.value)

        response = await client.setDevicePropValue(
            prop = prop,
            propType = propType,
            value = value
        )

        print(str(response))



import asyncio
import sys

from ptpip.client import PtpIpClient

class InitCaptureCommand():
    def __init__(self, filepath):
        super(InitCaptureCommand, self).__init__()

        self.filepath = filepath

    async def run(self, client: PtpIpClient):
        imgData = await client.initCapture()

        if self.filepath is not None:
            f = open(self.filepath, 'wb')
            f.write(imgData)
            f.close()
            print('Image saved in ' + str(self.filepath))
        else:
            sys.stdout.buffer.write(imgData)



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.cmd_type import CmdType
from ptpip.constants.property_type import PropertyType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_writer import StreamWriter

class CmdRequest(Packet):
    def __init__(
        self,
        transactionId = None,
        data = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        cmd = None,
        param1 = None,
        param2 = None,
        param3 = None,
        param4 = None,
        param5 = None,
        paramType1: PropertyType = PropertyType.Uint32,
        paramType2: PropertyType = PropertyType.Uint32,
        paramType3: PropertyType = PropertyType.Uint32,
        paramType4: PropertyType = PropertyType.Uint32,
        paramType5: PropertyType = PropertyType.Uint32,
    ):
        super(CmdRequest, self).__init__(
            PacketType.CmdRequest,
            transactionId = transactionId,
            data = data,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        self.cmd = CmdType(cmd)
        self.param1 = param1
        self.param2 = param2
        self.param3 = param3
        self.param4 = param4
        self.param5 = param5
        self.paramType1 = paramType1
        self.paramType2 = paramType2
        self.paramType3 = paramType3
        self.paramType4 = paramType4
        self.paramType5 = paramType5

    def pack(self):
        writer = StreamWriter() \
            .writeUint32(self.type.value) \
            .writeUint32(self.dataObjectTransferMode.value) \
            .writeUint16(self.cmd.value) \
            .writeUint32(self.transactionId)

        if self.param1 is not None:
            writer.writeType(self.paramType1.name, self.param1)
        if self.param2 is not None:
            writer.writeType(self.paramType2.name, self.param2)
        if self.param3 is not None:
            writer.writeType(self.paramType3.name, self.param3)
        if self.param4 is not None:
            writer.writeType(self.paramType4.name, self.param4)
        if self.param5 is not None:
            writer.writeType(self.paramType5.name, self.param5)

        return writer.data

    def __str__(self):
        return 'CmdRequest: ' + "\n" \
            + "\t" + 'type: ' + self.type.name + "\n" \
            + "\t" + 'dataObjectTransferMode: ' + str(self.dataObjectTransferMode.name) + "\n" \
            + "\t" + 'cmd: ' + str(self.cmd.name) + "\n" \
            + "\t" + 'param1: ' + str(self.param1) + "\n" \
            + "\t" + 'param2: ' + str(self.param2) + "\n" \
            + "\t" + 'param3: ' + str(self.param3) + "\n" \
            + "\t" + 'param4: ' + str(self.param4) + "\n" \
            + "\t" + 'param5: ' + str(self.param5) + "\n" \
            + "\t" + 'transactionId: ' + str(self.transactionId) + "\n"




from enum import Enum

class ExternalSpeedLightSort(Enum):
    CommunicationDisabled                               = 0x00
    Reserved                                            = 0x01
    NewTypeCommunicationWithSettingAndDisplaySection    = 0x02
    NewTypeCommunicationWithoutSettingAndDisplaySection = 0x03



from enum import Enum

# Indicates “AE lock timer – Self-timer – The number of captured frames”
# in the custom menu.
class SelfTimerShootExpose(Enum):
    Frame1 = 0x00
    Frame2 = 0x01
    Frame3 = 0x02
    Frame4 = 0x03
    Frame5 = 0x04
    Frame6 = 0x05
    Frame7 = 0x06
    Frame8 = 0x07
    Frame9 = 0x08



from ptpip.constants.object_format import ObjectFormat
from ptpip.constants.on_off_property import OnOffProperty
from ptpip.constants.property_type import PropertyType
from ptpip.constants.property_type_mutation import PropertyTypeMutation

from ptpip.constants.object.property_type import ObjectPropertyType

from ptpip.constants.object.image_bit_depth import ImageBitDepth
from ptpip.constants.object.number_of_channels import NumberOfChannels
from ptpip.constants.object.sample_rate import SampleRate
from ptpip.constants.object.scan_type import ScanType
from ptpip.constants.object.video_four_cc_code import VideoFourCCCode

from ptpip.packet.stream_reader import StreamReader

class ObjectPropDesc():
	def __init__(self, packet, data):
		super(self, ObjectPropDesc).__init__()

        self.packet = packet
        if data == None:
            return

        reader = StreamReader(data = data)

        self.typeId = reader.readUint16()
        self.type = ObjectPropertyType(self.typeId) \
            if self.typeId in ObjectPropertyType._value2member_map_ \
            else None

        self.propTypeId = reader.readUint16()
        self.propType = PropertyType(self.propTypeId) \
            if self.propTypeId in PropertyType._value2member_map_ \
            else None

        self.mode = ReadMode(reader.readUint8())

        if self.propType == None:
            return

        self.defaultValue = reader.readType(self.propType.name)
        self.value = reader.readType(self.propType.name)

        self.mutation = PropertyTypeMutation(reader.readUint8())

        self.minValue = None
        self.maxValue = None
        self.step = None
        self.values = None
        self.length = None
        self.maxLength = None

        try:
            if self.mutation == PropertyTypeMutation.Range:
                self.minValue = reader.readType(self.propType.name)
                self.maxValue = reader.readType(self.propType.name)
                self.step = reader.readType(self.propType.name)
            elif self.mutation == PropertyTypeMutation.Enumeration:
                self.values = reader.readArray(
                    self.propType.name,
                    lengthTypeName = PropertyType.Uint16.name
                )
            elif self.mutation == PropertyTypeMutation.Array:
            	self.length = reader.readUint16()
            elif self.mutation == PropertyTypeMutation.ByteString \
            	or self.mutation == PropertyTypeMutation.Text \
            :
            	self.maxLength = reader.readUint16()
        except Exception as e:
            print(str(e))

    def propertyValueStr(self, value):
        if self.type == ObjectPropertyType.ImageBitDepth \
            and value in ImageBitDepth._value2member_map_ \
            :
                return ImageBitDepth(value).name

        if self.type == ObjectPropertyType.NumberOfChannels \
            and value in NumberOfChannels._value2member_map_ \
            :
                return NumberOfChannels(value).name

        if self.type == ObjectPropertyType.ObjectFormat \
            and value in ObjectFormat._value2member_map_ \
            :
                return ObjectFormat(value).name

        if self.type == ObjectPropertyType.SampleRate \
            and value in SampleRate._value2member_map_ \
            :
                return SampleRate(value).name

        if self.type == ObjectPropertyType.ScanType \
            and value in ScanType._value2member_map_ \
            :
                return ScanType(value).name

        if self.type == ObjectPropertyType.VideoFourCCCode \
            and value in VideoFourCCCode._value2member_map_ \
            :
                return VideoFourCCCode(value).name

        if self.type in (
            ObjectPropertyType.ProtectionStatus
        ) and value in OnOffProperty._value2member_map_ \
        :
            return OnOffProperty(value).name

        return str(value)

    def __str__(self):
        sMutation = ''
        if self.mutation == PropertyTypeMutation.Range:
            if self.minValue != 0 and self.maxValue != 0:
                sMutation += "\t" \
                    + 'min_value: ' \
                        + self.propertyValueStr(self.minValue) + "\n" \
                    + "\t" \
                    + 'max_value: ' \
                        + self.propertyValueStr(self.maxValue) + "\n"

            if self.step != 1:
                sMutation += "\t" + 'step: ' + str(self.step) + "\n"
        elif self.mutation == PropertyTypeMutation.Enumeration:
            sValues = []
            for value in self.values:
                sValues.append(self.propertyValueStr(value))

            sMutation = "\t" + 'values: ' + str(sValues) + "\n"
        elif self.mutation == PropertyTypeMutation.Array:
        	sMutation += "\t" \
        		+ 'length: ' + str(self.length) + "\n"
        elif self.mutation == PropertyTypeMutation.ByteString \
        	or self.mutation == PropertyTypeMutation.Text \
        :
        	sMutation += "\t" \
        		+ 'maxLength: ' + str(self.maxLength) + "\n"

        return 'ObjectPropDesc: ' + "\n" \
            + "\t" + 'typeId: ' + str(self.typeId) + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'propTypeId: ' + str(self.propTypeId) + "\n" \
            + "\t" + 'propType: ' + str(self.propType) + "\n" \
            + "\t" + 'mode: ' + str(self.mode.name) + "\n" \
            + "\t" + 'defaultValue: ' \
                + self.propertyValueStr(self.defaultValue) + "\n" \
            + "\t" + 'value: ' \
                + self.propertyValueStr(self.value) + "\n" \
            + "\t" + 'mutation: ' + str(self.mutation) + "\n" \
            + sMutation



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader

class InitCmdAck(Packet):
    def __init__(
        self,
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        sessionId = None
    ):
        super(InitCmdAck, self).__init__(
            PacketType.InitCmdAck,
            data = data,
            transactionId = transactionId,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        self.sessionId = sessionId
        self.guid = None
        self.hostname = None

        if data is not None:
            reader = StreamReader(data = data)
            self.sessionId = reader.readUint32()
            self.guid = reader.readBytes(16)
            self.hostname = reader.readRest()

    def __str__(self):
        return 'InitCmdAck: ' + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'sessionId: ' + str(self.sessionId) + "\n" \
            + "\t" + 'guid: ' + str(self.guid) + "\n" \
            + "\t" + 'hostname: ' + str(self.hostname) + "\n"



import asyncio
import socket
import struct
import time

from ptpip.constants.cmd_type import CmdType
from ptpip.constants.property_type import PropertyType
from ptpip.constants.response_code import ResponseCode
from ptpip.constants.device.property_type import DevicePropertyType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject
from ptpip.data_object.device_info import DeviceInfo
from ptpip.data_object.device_prop_desc import DevicePropDesc
from ptpip.data_object.object_handle_array import ObjectHandleArray
from ptpip.data_object.storage_id_array import StorageIdArray
from ptpip.data_object.storage_info import StorageInfo

from ptpip.event.factory import EventFactory

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader
from ptpip.packet.stream_writer import StreamWriter
from ptpip.packet.factory import PacketFactory
from ptpip.packet.cmd_request import CmdRequest
from ptpip.packet.cmd_response import CmdResponse
from ptpip.packet.start_data import StartDataPacket
from ptpip.packet.data import DataPacket
from ptpip.packet.end_data import EndDataPacket
from ptpip.packet.event_req import EventReq
from ptpip.packet.init_cmd_ack import InitCmdAck
from ptpip.packet.init_cmd_req import InitCmdReq
from ptpip.packet.ping import Ping

class Connection():
    DEFAULT_HOST = '192.168.1.1'
    DEFAULT_PORT = 15740

    """docstring for PtpIP"""
    def __init__(self, debug = False):
        super(Connection, self).__init__()

        self.debug = debug
        self.session = None
        self.sessionEvents = None
        self.sessionId = None
        self.cmdQueue = []
        self.eventQueue = []
        self.objectQueue = []

        self.lastTransactionId = 2021;

    def createTransaction(self):
        self.lastTransactionId += 1

        return self.lastTransactionId

    def open(self,
        host = None,
        port = None,
        transactionId = None
    ):
        # Open both session, first one for for commands, second for events
        self.session = self.connect(host = host, port = port)
        self.sendReceivePacket(InitCmdReq(), self.session)

        self.sessionEvents = self.connect(host = host, port = port)
        self.sendReceivePacket(EventReq(), self.sessionEvents)

        cmd = CmdRequest(
            transactionId = transactionId \
                if transactionId != None \
                else self.createTransaction(),
            cmd = CmdType.OpenSession.value,
            param1 = self.sessionId,
            paramType1 = PropertyType.Uint32
        )

        self.sendReceivePacket(cmd, self.session)

    def communicationThread(self, delay = 0):
        try:
            while True:
                if len(self.cmdQueue) == 0:
                    # do a ping receive a pong (same as ping) as reply to keep the connection alive
                    # couldnt get any reply onto a propper Ping packet so i am querying the status
                    # of the device
                    """
                    reply = self.sendReceivePacket(
                        CmdRequest(
                            transactionId = self.createTransaction(),
                            cmd = CmdType.DeviceReady.value
                        ),
                        self.session
                    )
                    """
                else:
                    cmd = self.cmdQueue.pop()
                    reply = self.sendReceivePacket(cmd, self.session)
                    if self.debug \
                        and reply.code != ResponseCode.OK.value \
                        and reply.code != ResponseCode.DeviceBusy.value \
                    :
                        print("CCC Cmd reply is: " + str(reply.code))

                time.sleep(delay)
        except Exception as err:
            raise(Exception('Error in communication thread: ' + str(err)))

        print(str('End of communication'))

    def queueObject(self, dataObject):
        if not isinstance(dataObject.packet, CmdRequest):
            self.objectQueue.append(dataObject)
            return

        if dataObject.packet.cmd == CmdType.GetDeviceInfo:
            device = DeviceInfo(dataObject.packet, dataObject.data)
            self.objectQueue.append(device)
            return

        if dataObject.packet.cmd == CmdType.GetObject:
            self.objectQueue.append(dataObject)
            return

        if dataObject.packet.cmd == CmdType.GetEvent:
            events = EventFactory(dataObject.data).getEvents()

            for event in events:
                self.eventQueue.append(event)

            return

        if dataObject.packet.cmd == CmdType.GetStorageIDs:
            storageIds = StorageIdArray(dataObject.packet, dataObject.data)
            self.objectQueue.append(storageIds)
            return

        if dataObject.packet.cmd == CmdType.GetStorageInfo:
            storage = StorageInfo(dataObject.packet, dataObject.data)
            self.objectQueue.append(storage)
            return

        if dataObject.packet.cmd == CmdType.GetObjectHandles:
            handles = ObjectHandleArray(dataObject.packet, dataObject.data)
            self.objectQueue.append(handles)
            return

        if dataObject.packet.cmd == CmdType.GetDevicePropDesc \
            and dataObject.data != None \
        :
            devicePropDesc = DevicePropDesc(dataObject.packet, dataObject.data)
            self.objectQueue.append(devicePropDesc)

            return

        self.objectQueue.append(dataObject)

    async def listenObjectDataQueue(self, delay = 0):
        while True:
            # print('OOO Objects in queue: ' + str(len(self.objectQueue)))

            for idx, dataObject in enumerate(self.objectQueue):
                yield dataObject

            time.sleep(delay)
            pass

    async def listenEventQueue(self, delay = 0):
        if delay < 1:
            delay = 1

        while True:
            # print('OOO Events in queue: ' + str(len(self.eventQueue)))

            cmd = CmdRequest(
                cmd = CmdType.GetEvent.value,
                transactionId = self.createTransaction()
            )

            self.sendCmd(cmd)

            for idx, event in enumerate(self.eventQueue):
                yield event

            time.sleep(delay)
            pass

    def sendCmd(self, packet):
        self.cmdQueue.append(packet)

    def connect(self, host = None, port = None):
        if host == None:
            host = self.DEFAULT_HOST

        if port == None:
            port = self.DEFAULT_PORT

        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
            s.connect((host, port))
        except socket.error as err:
            if s:
                s.close()
            raise(Exception("Could not open socket: " + str(err)))
        return s

    def sendReceivePacket(self, packet: Packet, session):
        if isinstance(packet, EventReq) and packet.sessionId is None:
            packet.sessionId = self.sessionId

        self.sendPacket(packet, session)
        # print(str(packet.transactionId))

        # send data object
        if packet.dataObject != None \
            and (
                packet.dataObjectTransferMode
                    == DataObjectTransferMode.Send
                or packet.dataObjectTransferMode
                    == DataObjectTransferMode.SendAndReceive
            ) \
        :
            self.sendPacket(
                StartDataPacket(
                    transactionId = packet.transactionId,
                    length = len(packet.dataObject.data)
                ),
                session
            )
            self.sendPacket(
                DataPacket(
                    content = packet.dataObject.data,
                    transactionId = packet.transactionId
                ),
                session
            )
            self.sendPacket(
                EndDataPacket(
                    transactionId = packet.transactionId
                ),
                session
            )

        # receive response
        reply = self.receivePacket(session)

        if isinstance(reply, InitCmdAck):
            self.sessionId = reply.sessionId

        # receive data object
        if packet.dataObjectTransferMode \
            != DataObjectTransferMode.NoTransfer \
        :
            dataLength = 0
            if isinstance(reply, StartDataPacket):
                dataLength = reply.length

                reply = self.receivePacket(session, request = packet)
                data = reply.data
                while isinstance(reply, DataPacket):
                    data += reply.content
                    reply = self.receivePacket(session, request = packet)

                if dataLength == len(data):
                    self.queueObject(DataObject(packet, data = data))

                reply = self.receivePacket(session, request = packet)
            else:
                self.queueObject(DataObject(reply, data = None))

        return reply

    def sendEventReq(self, packet: Packet, session):
        # add the session id of the object itself if it is not specified in the package
        if packet.sessionId is None:
            packet.sessionId = self.sessionId

        self.sendPacket(packet, session)

    def sendPacket(self, packet, session):
        if self.debug:
            print('---- SEND ----')
            print(str(packet))
        self.sendData(packet.pack(), session)

        if self.debug:
            print('---- End SEND ----')

    def receivePacket(self, session, request: Packet = None):
        if self.debug:
            print('---- RECV ----')

        packet = PacketFactory.createPacket(
            data = self.receiveData(session),
            request = request,
            sessionId = self.sessionId
        )

        if self.debug:
            print(str(packet))
            print('---- End RECV ----')

        return packet

    def sendData(self, data, session):
        err = session.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if 0 != err:
            raise OSError('Socket error: ' + str(err))

        session.send(
            StreamWriter() \
                .writeUint32(len(data) + 4) \
                .writeBytes(data) \
                .data
        )

    def receiveData(self, session, nbTries = 3):
        if self.debug:
            print('RECV DATA')

        err = session.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if 0 != err:
            raise OSError('Socket error: ' + str(err))

        data = session.recv(4)

        if len(data) < 4:
            if nbTries > 1:
                self.receiveData(session, nbTries - 1)
            else:
                raise(Exception('Communication lost'))

        reader = StreamReader(data = data)
        dataLength = reader.readUint32()

        if self.debug:
            print("RECV Data length: " + str(dataLength))

        while dataLength > len(reader.data):
            reader.data += session.recv(dataLength - len(reader.data))

        return reader.readRest()



from ptpip.data_object.uint32_array import Uint32Array

class StorageIdArray(Uint32Array):
    def __init__(self, packet, data):
        super(StorageIdArray, self).__init__(packet, data)

        self.sdCardInserted = None

        if data is not None:
            self.sdCardInserted = len(self.elements) > 0 \
                and self.elements[0] == 0x00010001

    def __str__(self):
        return super.__str__(self) \
            + "\t" + 'sdCardInserted: ' + str(self.sdCardInserted) + "\n"



from enum import Enum

# Indicates the program shift value in units of 1/6 EV
# From -30 [-5 EV] to +30 [+5 EV]
# 20 or 30 values are valid epending on EVStep (1/2 or 1/3)
class FlexibleProgram(Enum):
    Minus30SixthEV  = -30
    Minus29SixthEV  = -29
    Minus28SixthEV  = -28
    Minus27SixthEV  = -27
    Minus26SixthEV  = -26
    Minus25SixthEV  = -25
    Minus24SixthEV  = -24
    Minus23SixthEV  = -23
    Minus22SixthEV  = -22
    Minus21SixthEV  = -21
    Minus20SixthEV  = -20
    Minus19SixthEV  = -19
    Minus18SixthEV  = -18
    Minus17SixthEV  = -17
    Minus16SixthEV  = -16
    Minus15SixthEV  = -15
    Minus14SixthEV  = -14
    Minus13SixthEV  = -13
    Minus12SixthEV  = -12
    Minus11SixthEV  = -11
    Minus10SixthEV  = -10
    Minus9SixthEV   = -9
    Minus8SixthEV   = -8
    Minus7SixthEV   = -7
    Minus6SixthEV   = -6
    Minus5SixthEV   = -5
    Minus4SixthEV   = -4
    Minus3SixthEV   = -3
    Minus2SixthEV   = -2
    Minus1SixthEV   = -1
    ZeroEV          = 0
    Plus1SixthEV    = 1
    Plus2SixthEV    = 2
    Plus3SixthEV    = 3
    Plus4SixthEV    = 4
    Plus5SixthEV    = 5
    Plus6SixthEV    = 6
    Plus7SixthEV    = 7
    Plus8SixthEV    = 8
    Plus9SixthEV    = 9
    Plus10SixthEV   = 10
    Plus11SixthEV   = 11
    Plus12SixthEV   = 12
    Plus13SixthEV   = 13
    Plus14SixthEV   = 14
    Plus15SixthEV   = 15
    Plus16SixthEV   = 16
    Plus17SixthEV   = 17
    Plus18SixthEV   = 18
    Plus19SixthEV   = 19
    Plus20SixthEV   = 20
    Plus21SixthEV   = 21
    Plus22SixthEV   = 22
    Plus23SixthEV   = 23
    Plus24SixthEV   = 24
    Plus25SixthEV   = 25
    Plus26SixthEV   = 26
    Plus27SixthEV   = 27
    Plus28SixthEV   = 28
    Plus29SixthEV   = 29
    Plus30SixthEV   = 30



from ptpip.constants.event_type import EventType

class Event():
    def __init__(self, typeId, parameter):
        super(Event, self).__init__()

        self.typeId = typeId
        self.type = EventType(self.typeId) \
            if self.typeId in EventType._value2member_map_ \
            else None

        self.parameter = parameter

    def __str__(self):
        return 'Event: ' + "\n" \
            + "\t" + 'typeId: ' + str(self.typeId) + "\n" \
            + "\t" + 'type: ' + (self.type.name if self.type != None else '') + "\n" \
            + "\t" + 'parameter: ' + str(self.parameter) + "\n"



from enum import Enum

class PacketType(Enum):
	InitCmdReq = 1
	InitCmdAck = 2
	EventReq = 3
	EventAck = 4
	InitFail = 5
	CmdRequest = 6
	CmdResponse = 7
	Event = 8
	StartData = 9
	Data = 10
	CancelTransaction = 11
	EndData = 12
	Ping = 13
	Pong = 14



from enum import Enum

class EventType(Enum):
    # PTP Event Code 0x4000..0x4FFF
    Undefined                   = 0x4000
    CancelTransaction           = 0x4001
    ObjectAdded                 = 0x4002
    ObjectRemoved               = 0x4003
    StoreAdded                  = 0x4004
    StoreRemoved                = 0x4005
    DevicePropChanged           = 0x4006
    ObjectInfoChanged           = 0x4007
    DeviceInfoChanged           = 0x4008
    RequestObjectTransfer       = 0x4009
    StoreFull                   = 0x400A
    DeviceReset                 = 0x400B
    StorageInfoChanged          = 0x400C
    CaptureComplete             = 0x400D
    UnreportedStatus            = 0x400E

    # Vendor Extension Event Code 0xC000..0xC7FF
    ObjectAddedInSdram          = 0xC101
    CaptureCompleteRecInSdram   = 0xC102
    RecordingInterrupted        = 0xC105

    # MTP Event Code 0xC800..0xC8FF
    ObjectPropChanged           = 0xC801
    ObjectPropDescChanged       = 0xC802
    ObjectReferenceChanged      = 0xC803



from enum import Enum

class DatacodeType(Enum):
    Undefined                       = 0x0000
    PTPOperationCode                = 0x1000
    PTPResponseCode                 = 0x2000
    PTPObjectFormatCode             = 0x3000
    PTPEventCode                    = 0x4000
    PTPDevicePropCode               = 0x5000
    ReservedPTP1                    = 0x6000
    ReservedPTP2                    = 0x7000
    Undefined2                      = 0x8000
    VendorExtensionOperationCode    = 0x9000
    MTPOperationCode                = 0x9800
    VendorExtensionResponseCode     = 0xA000
    MTPResponseCode                 = 0xA800
    VendorExtensionObjectFormatCode = 0xB000
    MTPObjectFormatCode             = 0xB800
    VendorExtensionEventCode        = 0xC000
    MTPEventCode                    = 0xC800
    VendorExtensionDevicePropCode   = 0xD000
    MTPDevicePropCode               = 0xD400
    VendorExtensionObjectPropCode   = 0xD800
    MTPObjectPropCode               = 0xDC00
    ReservedPTP3                    = 0xE000
    ReservedPTP4                    = 0xF000



from enum import Enum

class LiveViewProhibitionCondition(Enum):
    RecordingToSDCard = 1
    SequenceError = 2
    FullyPressedButtonError = 8
    ApertureValueSetByLensApertureRing = 16
    BulbError = 32
    DuringMirrorUpOperation = 64
    DuringInsufficiencyOfBattery = 128
    TTLError = 256
    WhileApertureValueOperationByLensApertureRingIsValid = 512
    NonCPULensMountedAndExposureModeIsNotManual = 1024



from enum import Enum

# Indicates “Shooting, recording, and display – Date imprint setting”
# in the custom menu.
class DateImprintSetting(Enum):
    Off                 = 0x00
    YearMonthDate       = 0x01
    YearMonthDateTime   = 0x02
    BirthdayCounter     = 0x03



import asyncio

from ptpip.constants.property_type import PropertyType
from ptpip.constants.device.property_type import DevicePropertyType

from ptpip.data_object.device_prop_desc import DevicePropDesc

from ptpip.client import PtpIpClient

class GetDevicePropValueCommand():
    def __init__(self, name, id, propTypeName, propTypeId):
        super(GetDevicePropValueCommand, self).__init__()

        self.name = name
        self.id = id
        self.propTypeName = propTypeName
        self.propTypeId = propTypeId

    async def run(self, client: PtpIpClient):
        prop = self.id if self.id != None else DevicePropertyType[self.name].value

        propType = None
        if self.propTypeId is not None:
            propType = PropertyType(self.propTypeId)
        elif self.propTypeName is not None:
            propType = PropertyType[self.propTypeName]
        else:
            propDesc = await client.getDevicePropDesc(
                prop = prop
            )

            if not isinstance(propDesc, DevicePropDesc):
                raise(Exception('Error while fetching prop desc: ' + str(propDesc)))

            propType = propDesc.propType

        if not isinstance(propType, PropertyType):
            raise(Exception('Property type not found'))

        propValue = await client.getDevicePropValue(
            prop = prop,
            propType = propType
        )

        print(str(propValue))



import asyncio

from ptpip.constants.object_format import ObjectFormat

from ptpip.data_object.object_handle_array import ObjectHandleArray

from ptpip.client import PtpIpClient

class GetObjectHandlesCommand():
    def __init__(
        self,
        storageId,
        objectFormatId,
        objectFormatName,
        parentHandle,
        allStorages,
        allFormats
    ):
        super(GetObjectHandlesCommand, self).__init__()

        self.storageId = storageId
        self.objectFormatId = objectFormatId
        self.objectFormatName = objectFormatName
        self.parentHandle = parentHandle
        self.allStorages = allStorages
        self.allFormats = allFormats

    async def run(self, client: PtpIpClient):
        objectFormat = self.objectFormatId
        if objectFormat is None \
            and self.objectFormatName is not None \
        :
            objectFormat = ObjectFormat[self.objectFormatName].value

        handles = await client.getObjectHandles(
            storageId = self.storageId,
            objectFormatId = objectFormat,
            handle = self.parentHandle,
            allStorages = self.allStorages,
            allFormats = self.allFormats,
        )

        if not isinstance(handles, ObjectHandleArray):
            raise(Exception('Error while fetching object handles: ' + str(handles)))

        for idx in range(0, len(handles.elements)):
            print(str(handles.elements[idx]))



import struct

from ptpip.constants.property_type import PropertyType

class StreamReader():
    def __init__(self, data = None, pos = 0):
        super(StreamReader, self).__init__()

        self.data = data
        self.pos = 0

    def readBytes(self, length = 1):
        pos2 = self.pos + length
        val = self.data[self.pos:pos2]
        self.pos = pos2
        return val

    def readRest(self):
        pos2 = len(self.data)
        val = self.data[self.pos:pos2]
        self.pos = pos2
        return val

    def readInt8(self):
        pos2 = self.pos + 1
        val = struct.unpack('b', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readUint8(self):
        pos2 = self.pos + 1
        val = struct.unpack('B', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readInt16(self):
        pos2 = self.pos + 2
        val = struct.unpack('h', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readUint16(self):
        pos2 = self.pos + 2
        val = struct.unpack('H', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readInt32(self):
        pos2 = self.pos + 4
        val = struct.unpack('i', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readUint32(self):
        pos2 = self.pos + 4
        val = struct.unpack('I', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readInt64(self):
        pos2 = self.pos + 8
        val = struct.unpack('q', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readUint64(self):
        pos2 = self.pos + 8
        val = struct.unpack('Q', self.data[self.pos:pos2])[0]
        self.pos = pos2
        return val

    def readString(self):
        length = self.readUint8()

        s = ''
        for i in range(0, length):
            c = self.readUint16()
            if c == 0:
                break

            s += chr(c)

        return s

    def readArrayByLength(self, typeLength):
        length = self.readUint32()

        results = []
        for i in range(0, length):
            pos2 = self.pos + typeLength
            results.append(self.data[self.pos:pos2])
            self.pos = pos2

        return results

    def readArray(self, typeName, lengthTypeName = PropertyType.Uint32.name):
        length = self.readType(lengthTypeName)

        results = []
        for i in range(0, length):
            results.append(
                self.readType(typeName)
            )

        return results

    def readInt8Array(self):
        return self.readArray(PropertyType.Int8.name)
    def readUint8Array(self):
        return self.readArray(PropertyType.Uint8.name)

    def readInt16Array(self):
        return self.readArray(PropertyType.Int16.name)
    def readUint16Array(self):
        return self.readArray(PropertyType.Uint16.name)

    def readInt32Array(self):
        return self.readArray(PropertyType.Int32.name)
    def readUint32Array(self):
        return self.readArray(PropertyType.Uint32.name)

    def readInt64Array(self):
        return self.readArray(PropertyType.Int64.name)
    def readUint64Array(self):
        return self.readArray(PropertyType.Uint64.name)

    def readType(self, typeName):
        if typeName == PropertyType.Int8.name:
            return self.readInt8()
        if typeName == PropertyType.Uint8.name:
            return self.readUint8()

        if typeName == PropertyType.Int16.name:
            return self.readInt16()
        if typeName == PropertyType.Uint16.name:
            return self.readUint16()

        if typeName == PropertyType.Int32.name:
            return self.readInt32()
        if typeName == PropertyType.Uint32.name:
            return self.readUint32()

        if typeName == PropertyType.Int64.name:
            return self.readInt64()
        if typeName == PropertyType.Uint64.name:
            return self.readUint64()

        if typeName == PropertyType.String.name:
            return self.readString()

        if typeName == PropertyType.Int8Array.name:
            return self.readInt8Array()
        if typeName == PropertyType.Uint8Array.name:
            return self.readUint8Array()

        if typeName == PropertyType.Int16Array.name:
            return self.readInt16Array()
        if typeName == PropertyType.Uint16Array.name:
            return self.readUint16Array()

        if typeName == PropertyType.Int32Array.name:
            return self.readInt32Array()
        if typeName == PropertyType.Uint32Array.name:
            return self.readUint32Array()

        if typeName == PropertyType.Int64Array.name:
            return self.readInt64Array()
        if typeName == PropertyType.Uint64Array.name:
            return self.readUint64Array()



from enum import Enum

class PropertyTypeMutation(Enum):
    Undefined   = 0x0
    Range       = 0x1
    Enumeration = 0x2
    Time        = 0x3
    Array       = 0x4
    RegExp      = 0x5
    ByteString  = 0x6
    Text        = 0x7



from enum import Enum

class FunctionButton(Enum):
    SelfTimerSetting                    = 0x00
    ReleaseModeSetting                  = 0x01
    ImageQualityModeAndImageSizeSetting = 0x02
    ISOSensitivitySetting               = 0x03
    WhiteBalanceSetting                 = 0x04
    ADLSetting                          = 0x05
    PlusRAWRecording                    = 0x06
    AutoBracketing                      = 0x07



from enum import Enum

class ObjectFormat(Enum):
    Any                             = 0x0000

    # PTP Object Format Code 0x3000..0x3FFF
    UndefinedAncilliary             = 0x3000
    Association                     = 0x3001
    Script                          = 0x3002
    Executable                      = 0x3003
    Text                            = 0x3004
    Html                            = 0x3005
    Dpof                            = 0x3006
    Aiff                            = 0x3007
    Wav                             = 0x3008
    Mp3                             = 0x3009
    Avi                             = 0x300A
    Mpeg                            = 0x300B
    Asf                             = 0x300C
    Qt                              = 0x300D

    UndefinedImage                  = 0x3800
    ExifJpeg                        = 0x3801
    TiffEp                          = 0x3802
    Flashpix                        = 0x3803
    Bmp                             = 0x3804
    Ciff                            = 0x3805
    Gif                             = 0x3807
    Jfif                            = 0x3808
    Pcd                             = 0x3809
    Pict                            = 0x380A
    Png                             = 0x380B
    Tiff                            = 0x380D
    TiffIt                          = 0x380E
    Jp2                             = 0x380F
    Jpx                             = 0x3810
    Dng                             = 0x3811
    Heif                            = 0x3812

    # Vendor Extension Object Format Code 0xB000..0xB7FF
    NetworkAssociation              = 0xB102
    RAW                             = 0xb101

    M4a                             = 0xB215
    Artist                          = 0xB218

    # MTP Object Format Code 0xB800..0xBFFF
    UndefinedFirmware               = 0xB800
    UndefinedFirmwareAndroid        = 0xB802
    WindowsImageFormat              = 0xB881

    UndefinedAudio                  = 0xB900
    Wma                             = 0xB901
    Ogg                             = 0xB902
    Aac                             = 0xB903
    Audible                         = 0xB904
    Flac                            = 0xB906

    UndefinedVideo                  = 0xB980
    Wmv                             = 0xB981
    Mp4                             = 0xB982
    Mp2                             = 0xB983
    _3gp                            = 0xB984

    UndefinedCollection             = 0xBA00
    AbstractMultimediaAlbum         = 0xBA01
    AbstractImageAlbum              = 0xBA02
    AbstractAudioAlbum              = 0xBA03
    AbstractVideoAlbum              = 0xBA04
    AbstractAVPlaylist              = 0xBA05
    AbstractContactGroup            = 0xBA06
    AbstractMessageFolder           = 0xBA07
    AbstractChapteredProduction     = 0xBA08
    AbstractAudioPlaylist           = 0xBA09
    AbstractVideoPlaylist           = 0xBA0a
    AbstractMediacast               = 0xBA0b
    WplPlaylist                     = 0xBA10
    M3uPlaylist                     = 0xBA11
    MplPlaylist                     = 0xBA12
    AsxPlaylist                     = 0xBA13
    PlsPlaylist                     = 0xBA14

    UndefinedDocument               = 0xBA80
    AbstractDocument                = 0xBA81
    XmlDocument                     = 0xBA82
    MicrosoftWordDocument           = 0xBA83
    MhtCompiledHtmlDocument         = 0xBA84
    MicrosoftExcelSpreadsheet       = 0xBA85
    MicrosoftPowerPointPresentation = 0xBA86

    UndefinedMessage                = 0xBB00
    AbstractMessage                 = 0xBB01
    UndefinedContact                = 0xBB80
    AbstractContact                 = 0xBB81
    VCard2                          = 0xBB82
    VCard3                          = 0xBB83

    UndefinedCalendarItem           = 0xBE00
    AbstractCalendarItem            = 0xBE01
    VCalendar1                      = 0xBE02
    VCalendar2                      = 0xBE03
    UndefinedWindowsExecutable      = 0xBE80

    All                             = 0xFFFF



from ptpip.constants.on_off_property import OnOffProperty
from ptpip.constants.property_type import PropertyType
from ptpip.constants.property_type_mutation import PropertyTypeMutation

from ptpip.constants.device.property_type import DevicePropertyType

from ptpip.constants.device.active_pic_ctrl_item import ActivePicCtrlItem
from ptpip.constants.device.ae_bracketing_step import AEBracketingStep
from ptpip.constants.device.ae_af_lock_setting import AEAFLockSetting
from ptpip.constants.device.af_mode_select import AFModeSelect
from ptpip.constants.device.auto_off_time import AutoOffTime
from ptpip.constants.device.auto_meter_off_delay import AutoMeterOffDelay
from ptpip.constants.device.bracketing_type import BracketingType
from ptpip.constants.device.beep import Beep
from ptpip.constants.device.command_dial_rotation import CommandDialRotation
from ptpip.constants.device.date_count_display_setting import DateCountDisplaySetting
from ptpip.constants.device.date_counter_select import DateCounterSelect
from ptpip.constants.device.date_imprint_setting import DateImprintSetting
from ptpip.constants.device.exposure_ev_step import ExposureEVStep
from ptpip.constants.device.exposure_display_status import ExposureDisplayStatus
from ptpip.constants.device.exposure_index import ExposureIndex
from ptpip.constants.device.exposure_indicate_status import ExposureIndicateStatus
from ptpip.constants.device.exposure_metering_mode import ExposureMeteringMode
from ptpip.constants.device.exposure_program_mode import ExposureProgramMode
from ptpip.constants.device.exposure_time import ExposureTime
from ptpip.constants.device.external_speed_light_sort import ExternalSpeedLightSort
from ptpip.constants.device.external_speed_light_status import ExternalSpeedLightStatus
from ptpip.constants.device.flash_compensation import FlashCompensation
from ptpip.constants.device.flash_mode import FlashMode
from ptpip.constants.device.flexible_program import FlexibleProgram
from ptpip.constants.device.focus_area import FocusArea
from ptpip.constants.device.focus_metering_mode import FocusMeteringMode
from ptpip.constants.device.focus_mode import FocusMode
from ptpip.constants.device.function_button import FunctionButton
from ptpip.constants.device.indicator_display import IndicatorDisplay
from ptpip.constants.device.internal_flash_compensation import InternalFlashCompensation
from ptpip.constants.device.internal_flash_manual import InternalFlashManual
from ptpip.constants.device.internal_flash_mode import InternalFlashMode
from ptpip.constants.device.internal_flash_status import InternalFlashStatus
from ptpip.constants.device.lcd_power_off import LCDPowerOff
from ptpip.constants.device.lens_sort import LensSort
from ptpip.constants.device.lens_type import LensType
from ptpip.constants.device.live_view_image_zoom_ratio import LiveViewImageZoomRatio
from ptpip.constants.device.live_view_prohibition_condition import LiveViewProhibitionCondition
from ptpip.constants.device.live_view_screen_display_setting import LiveViewScreenDisplaySetting
from ptpip.constants.device.new_external_speed_light_mode import NewExternalSpeedLightMode
from ptpip.constants.device.numbering_mode import NumberingMode
from ptpip.constants.device.orientation import Orientation
from ptpip.constants.device.read_mode import ReadMode
from ptpip.constants.device.recording_media import RecordingMedia
from ptpip.constants.device.remote_control_delay import RemoteControlDelay
from ptpip.constants.device.scene_mode import SceneMode
from ptpip.constants.device.self_timer_delay import SelfTimerDelay
from ptpip.constants.device.self_timer_shoot_expose import SelfTimerShootExpose
from ptpip.constants.device.shutter_speed import ShutterSpeed
from ptpip.constants.device.still_capture_mode import StillCaptureMode
from ptpip.constants.device.usb_speed import USBSpeed
from ptpip.constants.device.warning_status import WarningStatus
from ptpip.constants.device.wb_bracketing_step import WBBracketingStep
from ptpip.constants.device.wb_tune_fluorescent import WBTuneFluorescent
from ptpip.constants.device.white_balance import WhiteBalance

from ptpip.packet.stream_reader import StreamReader
from ptpip.data_object.data_object import DataObject

class DevicePropDesc():
    def __init__(self, packet, data):
        super(DevicePropDesc, self).__init__()

        self.packet = packet
        if data == None:
            return

        reader = StreamReader(data = data)

        self.typeId = reader.readUint16()
        self.type = DevicePropertyType(self.typeId) \
            if self.typeId in DevicePropertyType._value2member_map_ \
            else None

        self.propTypeId = reader.readUint16()
        self.propType = PropertyType(self.propTypeId) \
            if self.propTypeId in PropertyType._value2member_map_ \
            else None

        self.mode = ReadMode(reader.readUint8())

        if self.propType == None:
            return

        self.defaultValue = reader.readType(self.propType.name)
        self.value = reader.readType(self.propType.name)

        self.mutation = PropertyTypeMutation(reader.readUint8())

        self.minValue = None
        self.maxValue = None
        self.step = None
        self.values = None

        try:
            if self.mutation == PropertyTypeMutation.Range:
                self.minValue = reader.readType(self.propType.name)
                self.maxValue = reader.readType(self.propType.name)
                self.step = reader.readType(self.propType.name)
            elif self.mutation == PropertyTypeMutation.Enumeration:
                self.values = reader.readArray(
                    self.propType.name,
                    lengthTypeName = PropertyType.Uint16.name
                )
        except Exception as e:
            print(str(e))

    def propertyValueStr(self, value):
        if self.type == DevicePropertyType.ActivePicCtrlItem \
            and value in ActivePicCtrlItem._value2member_map_ \
            :
                return ActivePicCtrlItem(value).name

        if self.type == DevicePropertyType.AEBracketingStep \
            and value in AEBracketingStep._value2member_map_ \
            :
                return AEBracketingStep(value).name

        if self.type == DevicePropertyType.AEAFLockSetting \
            and value in AEAFLockSetting._value2member_map_ \
            :
                return AEAFLockSetting(value).name

        if self.type == DevicePropertyType.AFModeSelect \
            and value in AFModeSelect._value2member_map_ \
            :
                return AFModeSelect(value).name

        if self.type == DevicePropertyType.AutoMeterOffDelay \
            and value in AutoMeterOffDelay._value2member_map_ \
            :
                return AutoMeterOffDelay(value).name

        if self.type == DevicePropertyType.AutoOffTime \
            and value in AutoOffTime._value2member_map_ \
            :
                return AutoOffTime(value).name

        if self.type == DevicePropertyType.BracketingType \
            and value in BracketingType._value2member_map_ \
            :
                return BracketingType(value).name

        if self.type == DevicePropertyType.Beep \
            and value in Beep._value2member_map_ \
            :
                return Beep(value).name

        if self.type == DevicePropertyType.CommandDialRotation \
            and value in CommandDialRotation._value2member_map_ \
            :
                return CommandDialRotation(value).name

        if self.type == DevicePropertyType.DateCountDisplaySetting \
            and value in DateCountDisplaySetting._value2member_map_ \
            :
                return DateCountDisplaySetting(value).name

        if self.type == DevicePropertyType.DateCounterSelect \
            and value in DateCounterSelect._value2member_map_ \
            :
                return DateCounterSelect(value).name

        if self.type == DevicePropertyType.DateImprintSetting \
            and value in DateImprintSetting._value2member_map_ \
            :
                return DateImprintSetting(value).name

        if self.type == DevicePropertyType.ExposureEVStep \
            and value in ExposureEVStep._value2member_map_ \
            :
                return ExposureEVStep(value).name

        if self.type == DevicePropertyType.ExposureDisplayStatus \
            and value in ExposureDisplayStatus._value2member_map_ \
            :
                return ExposureDisplayStatus(value).name

        if self.type == DevicePropertyType.ExposureIndex \
            and value in ExposureIndex._value2member_map_ \
            :
                return ExposureIndex(value).name

        if self.type == DevicePropertyType.ExposureIndicateStatus \
            and value in ExposureIndicateStatus._value2member_map_ \
            :
                return ExposureIndicateStatus(value).name

        if self.type == DevicePropertyType.ExposureMeteringMode \
            and value in ExposureMeteringMode._value2member_map_ \
            :
                return ExposureMeteringMode(value).name

        if self.type == DevicePropertyType.ExposureProgramMode \
            and value in ExposureProgramMode._value2member_map_ \
            :
                return ExposureProgramMode(value).name

        if self.type == DevicePropertyType.ExposureTime \
            and value in ExposureTime._value2member_map_ \
            :
                return ExposureTime(value).name

        if self.type == DevicePropertyType.ExternalSpeedLightSort \
            and value in ExternalSpeedLightSort._value2member_map_ \
            :
                return ExternalSpeedLightSort(value).name

        if self.type == DevicePropertyType.ExternalSpeedLightStatus \
            and value in ExternalSpeedLightStatus._value2member_map_ \
            :
                return ExternalSpeedLightStatus(value).name

        if self.type == DevicePropertyType.FlashMode \
            and value in FlashMode._value2member_map_ \
            :
                return FlashMode(value).name

        if self.type == DevicePropertyType.FlashCompensation \
            and value in FlashCompensation._value2member_map_ \
            :
                return FlashCompensation(value).name

        if self.type == DevicePropertyType.FlexibleProgram \
            and value in FlexibleProgram._value2member_map_ \
            :
                return FlexibleProgram(value).name

        if self.type == DevicePropertyType.FocusArea \
            and value in FocusArea._value2member_map_ \
            :
                return FocusArea(value).name

        if self.type == DevicePropertyType.FocusMeteringMode \
            and value in FocusMeteringMode._value2member_map_ \
            :
                return FocusMeteringMode(value).name

        if self.type == DevicePropertyType.FocusMode \
            and value in FocusMode._value2member_map_ \
            :
                return FocusMode(value).name

        if self.type == DevicePropertyType.FunctionButton \
            and value in FunctionButton._value2member_map_ \
            :
                return FunctionButton(value).name

        if self.type == DevicePropertyType.IndicatorDisplay \
            and value in IndicatorDisplay._value2member_map_ \
            :
                return IndicatorDisplay(value).name

        if self.type == DevicePropertyType.InternalFlashCompensation \
            and value in InternalFlashCompensation._value2member_map_ \
            :
                return InternalFlashCompensation(value).name

        if self.type == DevicePropertyType.InternalFlashManual \
            and value in InternalFlashManual._value2member_map_ \
            :
                return InternalFlashManual(value).name

        if self.type == DevicePropertyType.InternalFlashMode \
            and value in InternalFlashMode._value2member_map_ \
            :
                return InternalFlashMode(value).name

        if self.type == DevicePropertyType.InternalFlashStatus \
            and value in InternalFlashStatus._value2member_map_ \
            :
                return InternalFlashStatus(value).name

        if self.type == DevicePropertyType.ISOAutoHighLimit \
            and value in ExposureIndex._value2member_map_ \
            :
                return ExposureIndex(value).name

        if self.type == DevicePropertyType.ISOAutoShutterTime \
            and value in ExposureTime._value2member_map_ \
            :
                return ExposureTime(value).name

        if self.type == DevicePropertyType.LCDPowerOff \
            and value in LCDPowerOff._value2member_map_ \
            :
                return LCDPowerOff(value).name

        if self.type == DevicePropertyType.LensSort \
            and value in LensSort._value2member_map_ \
            :
                return LensSort(value).name

        if self.type == DevicePropertyType.LensType \
            and value in LensType._value2member_map_ \
            :
                return LensType(value).name

        if self.type == DevicePropertyType.LiveViewImageZoomRatio \
            and value in LiveViewImageZoomRatio._value2member_map_ \
            :
                return LiveViewImageZoomRatio(value).name

        if self.type == DevicePropertyType.LiveViewProhibitionCondition \
            and value in LiveViewProhibitionCondition._value2member_map_ \
            :
                return LiveViewProhibitionCondition(value).name

        if self.type == DevicePropertyType.LiveViewScreenDisplaySetting \
            and value in LiveViewScreenDisplaySetting._value2member_map_ \
            :
                return LiveViewScreenDisplaySetting(value).name

        if self.type == DevicePropertyType.NewExternalSpeedLightMode \
            and value in NewExternalSpeedLightMode._value2member_map_ \
            :
                return NewExternalSpeedLightMode(value).name

        if self.type == DevicePropertyType.NumberingMode \
            and value in NumberingMode._value2member_map_ \
            :
                return NumberingMode(value).name

        if self.type == DevicePropertyType.Orientation \
            and value in Orientation._value2member_map_ \
            :
                return Orientation(value).name

        if self.type == DevicePropertyType.RecordingMedia \
            and value in RecordingMedia._value2member_map_ \
            :
                return RecordingMedia(value).name

        if self.type == DevicePropertyType.RemoteControlDelay \
            and value in RemoteControlDelay._value2member_map_ \
            :
                return RemoteControlDelay(value).name

        if self.type == DevicePropertyType.SceneMode \
            and value in SceneMode._value2member_map_ \
            :
                return SceneMode(value).name

        if self.type == DevicePropertyType.SelfTimerDelay \
            and value in SelfTimerDelay._value2member_map_ \
            :
                return SelfTimerDelay(value).name

        if self.type == DevicePropertyType.SelfTimerShootExpose \
            and value in SelfTimerShootExpose._value2member_map_ \
            :
                return SelfTimerShootExpose(value).name

        if self.type == DevicePropertyType.ShutterSpeed \
            and value in ShutterSpeed._value2member_map_ \
            :
                return ShutterSpeed(value).name

        if self.type == DevicePropertyType.ShutterSpeed2 \
            and value in ShutterSpeed._value2member_map_ \
            :
                return ShutterSpeed(value).name

        if self.type == DevicePropertyType.StillCaptureMode \
            and value in StillCaptureMode._value2member_map_ \
            :
                return StillCaptureMode(value).name

        if self.type == DevicePropertyType.USBSpeed \
            and value in USBSpeed._value2member_map_ \
            :
                return USBSpeed(value).name

        if self.type == DevicePropertyType.WarningStatus \
            and value in WarningStatus._value2member_map_ \
            :
                return WarningStatus(value).name

        if self.type == DevicePropertyType.WBBracketingStep \
            and value in WBBracketingStep._value2member_map_ \
            :
                return WBBracketingStep(value).name

        if self.type == DevicePropertyType.WBTuneFluorescent \
            and value in WBTuneFluorescent._value2member_map_ \
            :
                return WBTuneFluorescent(value).name

        if self.type == DevicePropertyType.WhiteBalance \
            and value in WhiteBalance._value2member_map_ \
            :
                return WhiteBalance(value).name

        if self.type in (
            DevicePropertyType.AELockRelease,
            DevicePropertyType.AELockStatus,
            DevicePropertyType.AFLockStatus,
            DevicePropertyType.EnableBracketing,
            DevicePropertyType.EnableComment,
            DevicePropertyType.EnableShutter,
            DevicePropertyType.ExposureDelay,
            DevicePropertyType.ExposureIndicateLightup,
            DevicePropertyType.ExternalDCIn,
            DevicePropertyType.ExternalSpeedLightExist,
            DevicePropertyType.FinderISODisplay,
            DevicePropertyType.GridDisplay,
            DevicePropertyType.InfoDisplayErrorStatus,
            DevicePropertyType.InternalFlashPopup,
            DevicePropertyType.ISOAutoControl,
            DevicePropertyType.LiveViewStatus,
            DevicePropertyType.OrientationSensorMode
        ) and value in OnOffProperty._value2member_map_ \
        :
            return OnOffProperty(value).name

        return str(value)

    def __str__(self):
        sMutation = ''
        if self.mutation == PropertyTypeMutation.Range:
            if self.minValue != 0 and self.maxValue != 0:
                sMutation += "\t" \
                    + 'min_value: ' \
                        + self.propertyValueStr(self.minValue) + "\n" \
                    + "\t" \
                    + 'max_value: ' \
                        + self.propertyValueStr(self.maxValue) + "\n"

            if self.step != 1:
                sMutation += "\t" + 'step: ' + str(self.step) + "\n"
        elif self.mutation == PropertyTypeMutation.Enumeration:
            sValues = []
            for value in self.values:
                sValues.append(self.propertyValueStr(value))

            sMutation = "\t" + 'values: ' + str(sValues) + "\n"

        return 'DevicePropDesc: ' + "\n" \
            + "\t" + 'typeId: ' + str(self.typeId) + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'propTypeId: ' + str(self.propTypeId) + "\n" \
            + "\t" + 'propType: ' + str(self.propType) + "\n" \
            + "\t" + 'mode: ' + str(self.mode.name) + "\n" \
            + "\t" + 'defaultValue: ' \
                + self.propertyValueStr(self.defaultValue) + "\n" \
            + "\t" + 'value: ' \
                + self.propertyValueStr(self.value) + "\n" \
            + "\t" + 'mutation: ' + str(self.mutation) + "\n" \
            + sMutation



from enum import Enum

class AEBracketingStep(Enum):
    OneThirdEV      = 0x00
    OneHalfEV       = 0x01
    TwoHalvesEV     = 0x02
    OneEV           = 0x03
    FourThirdsEV    = 0x04
    ThreeHalvesEV   = 0x05
    FiveThirdsEV    = 0x06
    TwoEV           = 0x07



from enum import Enum

class DevicePropertyType(Enum):
    # PTP Device Prop Code 0x5000..0x5FFF
    Undefined                       = 0x5000
    BatteryLevel                    = 0x5001
    FunctionalMode                  = 0x5002 # @see FunctionalMode
    ImageSize                       = 0x5003
    CompressionSetting              = 0x5004
    WhiteBalance                    = 0x5005 # @see WhiteBalance
    RGBGain                         = 0x5006
    FNumber                         = 0x5007
    FocalLength                     = 0x5008
    FocusDistance                   = 0x5009
    FocusMode                       = 0x500A # @see FocusMode
    ExposureMeteringMode            = 0x500B # @see ExposureMeteringMode
    FlashMode                       = 0x500C # @see FlashMode
    ExposureTime                    = 0x500D # @see ExposureTime
    ExposureProgramMode             = 0x500E # @see ExposureProgramMode
    ExposureIndex                   = 0x500F # @see ExposureIndex
    ExposureBiasCompensation        = 0x5010
    DateTime                        = 0x5011
    CaptureDelay                    = 0x5012
    StillCaptureMode                = 0x5013 # @see StillCaptureMode
    Contrast                        = 0x5014
    Sharpness                       = 0x5015
    DigitalZoom                     = 0x5016
    EffectMode                      = 0x5017
    BurstNumber                     = 0x5018
    BurstInterval                   = 0x5019
    TimelapseNumber                 = 0x501A
    TimelapseInterval               = 0x501B
    FocusMeteringMode               = 0x501C # @see FocusMeteringMode
    UploadUrl                       = 0x501D
    Artist                          = 0x501E
    CopyrightInfo                   = 0x501F

    AudioVolume                     = 0x502C

    # Vendor Extension Device Prop Code 0xD000..0xD3FF
    ErrorInfo                       = 0xD006
    ShutterSpeed                    = 0xD00F # @see ShutterSpeed

    WBTuneAuto                      = 0xD017
    WBTuneIncandescent              = 0xD018
    WBTuneFluorescent               = 0xD019 # @see WBTuneFluorescent
    WBTuneSunny                     = 0xD01A
    WBTuneFlash                     = 0xD01B
    WBTuneCloudy                    = 0xD01C
    WBTuneShade                     = 0xD01D
    WBPresetDataNo                  = 0xD01F
    WBPresetDataValue0              = 0xD025
    WBPresetDataValue1              = 0xD026
    ColorSpace                      = 0xD032
    ResetCustomSetting              = 0xD045
    ISOAutoControl                  = 0xD054 # @see OnOffProperty
    ExposureEVStep                  = 0xD056 # @see ExposureEVStep
    AfAtLiveView                    = 0xD05D
    AELockRelease                   = 0xD05E # @see OnOffProperty
    AEAFLockSetting                 = 0xD05F # @see AEAFLockSetting
    AutoMeterOffDelay               = 0xD062 # @see AutoMeterOffDelay
    SelfTimerDelay                  = 0xD063 # @see SelfTimerDelay
    LCDPowerOff                     = 0xD064 # @see LCDPowerOff
    ImageConfirmTimeAfterPhoto      = 0xD065
    AutoOffTime                     = 0xD066 # @see AutoOffTime
    ExposureDelay                   = 0xD06A # @see ExposureDelay
    NoiseReduction                  = 0xD06B
    NumberingMode                   = 0xD06C # @see NumberingMode
    NoiseReductionHiIso             = 0xD070
    BracketingType                  = 0xD078 # @see BracketingType
    FunctionButton                  = 0xD084 # @see FunctionButton
    CommandDialRotation             = 0xD085 # @see CommandDialRotation
    EnableShutter                   = 0xD08A # @see OnOffProperty
    CommentString                   = 0xD090
    EnableComment                   = 0xD091 # @see OnOffProperty
    OrientationSensorMode           = 0xD092 # @see OnOffProperty
    MovieRecordScreenSize           = 0xD0A0
    MovieRecordWithVoice            = 0xD0A1
    LiveViewScreenDisplaySetting    = 0xD0B2 # @see LiveViewScreenDisplaySetting
    EnableBracketing                = 0xD0C0 # @see OnOffProperty
    AEBracketingStep                = 0xD0C1 # @see AEBracketingStep
    AEBracketingCount               = 0xD0C3
    WBBracketingStep                = 0xD0C4 # @see WBBracketingStep
    LensID                          = 0xD0E0
    LensSort                        = 0xD0E1 # @see LensSort
    LensType                        = 0xD0E2 # @see LensType
    LensFocalMin                    = 0xD0E3
    LensFocalMax                    = 0xD0E4
    LensApatureMin                  = 0xD0E5
    LensApatureMax                  = 0xD0E6
    FinderISODisplay                = 0xD0F0 # @see OnOffProperty
    SelfTimerShootExpose            = 0xD0F5 # @see SelfTimerShootExpose
    AutoDistortion                  = 0xD0F8
    SceneMode                       = 0xD0F9
    ShutterSpeed2                   = 0xD100 # @see ShutterSpeed
    ExternalDCIn                    = 0xD101 # @see OnOffProperty
    WarningStatus                   = 0xD102 # @see WarningStatus
    RemainingExposure               = 0xD103
    AFLockStatus                    = 0xD104 # @see OnOffProperty
    AELockStatus                    = 0xD105 # @see OnOffProperty
    FocusArea                       = 0xD108 # @see FocusArea
    FlexibleProgram                 = 0xD109 # @see FlexibleProgram
    RecordingMedia                  = 0xD10B # @see RecordingMedia
    USBSpeed                        = 0xD10C
    CCDNumber                       = 0xD10D
    Orientation                     = 0xD10E # @see Orientation
    ExternalSpeedLightExist         = 0xD120 # @see OnOffProperty
    ExternalSpeedLightStatus        = 0xD121 # @see ExternalSpeedLightStatus
    ExternalSpeedLightSort          = 0xD122 # @see ExternalSpeedLightSort
    FlashCompensation               = 0xD124 # @see FlashCompensation
    NewExternalSpeedLightMode       = 0xD125 # @see NewExternalSpeedLightMode
    InternalFlashCompensation       = 0xD126 # @see InternalFlashCompensation
    ActiveDLighting                 = 0xD14E
    WBTuneFluorescentType           = 0xD14F
    Beep                            = 0xD160 # @see Beep
    AFModeSelect                    = 0xD161 # @see AFModeSelect
    AFSubLight                      = 0xD163
    ISOAutoShutterTime              = 0xD164 # @see ExposureTime
    InternalFlashMode               = 0xD167 # @see InternalFlashMode
    ISOAutoSetting                  = 0xD16A
    RemoteControlDelay              = 0xD16B # @see RemoteControlDelay
    GridDisplay                     = 0xD16C # @see OnOffProperty
    InternalFlashManual             = 0xD16D # @see InternalFlashManual
    DateImprintSetting              = 0xD170 # @see DateImprintSetting
    DateCounterSelect               = 0xD171 # @see DateCounterSelect
    DateCountData                   = 0xD172
    DateCountDisplaySetting         = 0xD173 # @see DateCountDisplaySetting
    RangeFinderSetting              = 0xD174
    ISOAutoHighLimit                = 0xD183 # @see ExposureIndex
    IndicatorDisplay                = 0xD18D # @see IndicatorDisplay
    LiveViewStatus                  = 0xD1A2 # @see OnOffProperty
    LiveViewImageZoomRatio          = 0xD1A3 # @see LiveViewImageZoomRatio
    LiveViewProhibitionCondition    = 0xD1A4 # @see LiveViewProhibitionCondition
    ExposureDisplayStatus           = 0xD1B0 # @see ExposureDisplayStatus
    ExposureIndicateStatus          = 0xD1B1 # @see ExposureIndicateStatus
    InfoDisplayErrorStatus          = 0xD1B2 # @see OnOffProperty
    ExposureIndicateLightup         = 0xD1B3 # @see OnOffProperty
    InternalFlashPopup              = 0xD1C0 # @see OnOffProperty
    InternalFlashStatus             = 0xD1C1 # @see InternalFlashStatus

    ActivePicCtrlItem               = 0xD200 # @see ActivePicCtrlItem
    ChangePicCtrlItem               = 0xD201
    DeviceEUI64                     = 0xD210

    FunctionalID                    = 0xD301
    ModelID                         = 0xD302
    UseDeviceStageFlag              = 0xD303

    # MTP Device Prop Code 0xD400..0xD7FF
    SynchronizationPartner          = 0xD401
    DeviceFriendlyName              = 0xD402
    Volume                          = 0xD403
    SupportedFormatsOrdered         = 0xD404
    DeviceIcon                      = 0xD405
    SessionInitiatorVersionInfo     = 0xD406
    PerceivedDeviceType             = 0xD407
    PlaybackRate                    = 0xD410
    PlaybackObject                  = 0xD411
    PlaybackContainerIndex          = 0xD412
    PlaybackPosition                = 0xD413

    GpsInfo                         = 0xD801
    AutoPowerOffDelay               = 0xD802
    SleepDelay                      = 0xD803
    ChannelNumber                   = 0xD807
    CaptureStatus                   = 0xD808
    RecordingTime                   = 0xD809
    RemainingRecordingTime          = 0xD80A



from enum import Enum

class LiveViewImageZoomRatio(Enum):
    EntireDisplay   = 0
    Percent25       = 1
    Percent33       = 2
    Percent50       = 3
    Percent67       = 4
    Percent100      = 5



import asyncio

from ptpip.data_object.storage_id_array import StorageIdArray

from ptpip.client import PtpIpClient

class GetStorageIdsCommand():
    def __init__(self):
        super(GetStorageIdsCommand, self).__init__()

    async def run(self, client: PtpIpClient):
        storageIds = await client.getStorageIds()

        if not isinstance(storageIds, StorageIdArray):
            raise(Exception('Error while fetching storage ids: ' + str(storageIds)))

        for idx in range(0, len(storageIds.elements)):
            print(str(storageIds.elements[idx]))



from enum import Enum

class USBSpeed(Enum):
    FullSpeed       = 0 # 12 Mbit/s USB 1.0
    HighSpeed       = 1 # 480 Mbit/s USB 1.x and USB 2.0
    SuperSpeed      = 2 # 5.0 Gbit/s USB 3.0 (Unofficial value)
    SuperSpeedPlus  = 3 # 10 Gbit/s USB 3.1 (Unofficial value)



from datetime import datetime

from html import escape
from typing import List

from ptpip.constants.property_type_mutation import PropertyTypeMutation

from ptpip.data_object.device_info import DeviceInfo
from ptpip.data_object.device_prop_desc import DevicePropDesc

class HtmlDeviceReportGenerator():
    def __init__(
        self,
        device: DeviceInfo,
        props: List[DevicePropDesc] = [],
        discoveredProps: List[DevicePropDesc] = []
    ):
        super(HtmlDeviceReportGenerator, self).__init__()

        self.device = device
        self.props = props
        self.discoveredProps = discoveredProps

    def generateSupportedOperationsReport(self):
        html = '<ul>'

        for op in self.device.operations:
            html += '<li>' + escape(self.device.operationStr(op)) + '</li>'

        return html + '</ul>'


    def generateSupportedEventsReport(self):
        html = '<ul>'

        for evt in self.device.events:
            html += '<li>' + escape(self.device.eventStr(evt)) + '</li>'

        return html + '</ul>'

    def generateObjectFormatsReport(self, objectFormats):
        html = '<ul>'

        for objf in objectFormats:
            html += '<li>' + escape(self.device.objectFormatStr(objf)) + '</li>'

        return html + '</ul>'

    def generatePropertyReport(self, prop: DevicePropDesc):
        html = '<tr>' \
            + '<td style="vertical-align:top;">' \
                + escape(str(prop.typeId)) \
            + '</td>' \
            + '<td style="vertical-align:top;font-weight:bold;">' \
                + escape(prop.type.name if prop.type != None else '') \
            + '</td>' \
            + '<td>' \
                + escape(
                    prop.propType.name \
                    if prop.propType != None \
                    else str(prop.propTypeId)
                ) \
            + '</td>' \
            + '<td>' + escape(prop.mode.name) + '</td>' \
            + '<td>' \
                + escape(prop.propertyValueStr(prop.defaultValue)) \
            + '</td>' \
            + '<td>' \
                + escape(prop.propertyValueStr(prop.value)) \
            + '</td>' \
            + '<td>' \
                + escape(prop.mutation.name) \
            + '</td>'

        if prop.mutation == PropertyTypeMutation.Range:
            html += '<td>' \
                    + '<table border="1" width="100%">' \
                        + '<thead>' \
                            + '<th>Min</th>' \
                            + '<th>Max</th>' \
                            + '<th>Step</th>' \
                        + '</thead>' \
                        + '<tbody>' \
                            + '<tr>' \
                                + '<td>' \
                                    + escape(
                                        prop.propertyValueStr(prop.minValue)
                                    ) \
                                + '</td>' \
                                + '<td>' \
                                    + escape(
                                        prop.propertyValueStr(prop.maxValue)
                                    ) \
                                + '</td>' \
                                + '<td>' + escape(str(prop.step)) + '</td>' \
                            + '</tr>' \
                        + '</tbody>' \
                    + '</table>' \
                + '</td>'
        elif prop.mutation == PropertyTypeMutation.Enumeration:
            html += '<td>' \
                + '<ul>'

            for value in prop.values:
                html += '<li>' + escape(prop.propertyValueStr(value)) + '</li>'

            html += '</ul>' \
                + '</td>'
        else:
            html += '<td></td>'

        return html + '</tr>'

    def generatePropertiesReport(self, properties: List[DevicePropDesc] = []):
        html = '<table border="1" width="100%">' \
            + '<thead>' \
                + '<th>Id</th>' \
                + '<th>Name</th>' \
                + '<th>Type</th>' \
                + '<th>Mode</th>' \
                + '<th>Default</th>' \
                + '<th>Value</th>' \
                + '<th colspan="2">Boundaries</th>' \
            + '</thead>' \
            + '<tbody>'

        for prop in properties:
            html += self.generatePropertyReport(prop)

        return html + '</tbody>' \
            + '</table>'

    def generate(self):
        return '<table border="1">' \
            + '<thead><th>Key</th><th>Value</th></thead>' \
            + '<tbody>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Report date</td><td>' \
                    + escape(str(datetime.today().strftime('%Y-%m-%d %H:%M:%S'))) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Manufacturer</td><td>' \
                    + escape(str(self.device.manufacturer)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Model</td><td>' \
                + escape(str(self.device.model)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Version</td><td>' \
                    + escape(str(self.device.deviceVersion)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">serialNumber</td><td>' \
                    + escape(str(self.device.serialNumber)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Standard version</td><td>' \
                    + escape(str(self.device.standardVersion)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Vendor extension id</td><td>' \
                    + escape(self.device.vendorExtensionIdStr()) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Vendor extension version</td><td>' \
                    + escape(str(self.device.vendorExtensionVersion)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Vendor extension description</td><td>' \
                    + escape(str(self.device.vendorExtensionDesc)) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Status</td><td>' \
                    + escape(self.device.functionalModeStr()) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Operations</td><td>' \
                    + self.generateSupportedOperationsReport() \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Event types</td><td>' \
                    + self.generateSupportedEventsReport() \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Capture formats</td><td>' \
                    + self.generateObjectFormatsReport(self.device.captureFormats) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Image formats</td><td>' \
                    + self.generateObjectFormatsReport(self.device.imageFormats) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Official properties</td><td>' \
                    + self.generatePropertiesReport(self.props) \
                + '</td></tr>' \
                + '<tr><td style="vertical-align:top;font-weight:bold;">Discovered properties</td><td>' \
                    + self.generatePropertiesReport(self.discoveredProps) \
                + '</td></tr>' \
            + '</tbody>' \
            + '</table>'




from enum import Enum

# Indicates “Operation – Function of the AE-L/AF-L button”
# in the custom menu.
class AEAFLockSetting(Enum):
    AELAndAFL   = 0x00
    AEL         = 0x01
    AFL         = 0x02
    AELHold     = 0x03
    AFOn        = 0x04



from enum import Enum

class FocusMode(Enum):
    Undefined              = 0x0000
    ManualFocus            = 0x0001

    SingleAFServo          = 0x8010
    ContinuousAFServo      = 0x8011
    AFServoModeAutoSwitch  = 0x8012



from enum import Enum

class ExposureDisplayStatus(Enum):
    NormalApertureAndNormalShutterSpeed = 0x00
    NormalApertureAndLowShutterSpeed = 0x01
    NormalApertureAndHighShutterSpeed = 0x02
    LowApertureAndNormalShutterSpeed = 0x03
    LowApertureAndLowShutterSpeed = 0x04
    LowApertureAndHighShutterSpeed = 0x05
    HighApertureAndNormalShutterSpeed = 0x06
    HighApertureAndLowShutterSpeed = 0x07
    HighApertureAndHighShutterSpeed = 0x08



from enum import Enum

# Indicates the recording destination of the images captured
# by using the shutter-release button of the camera.
class RecordingMedia(Enum):
    SDCard  = 0x00
    SDRAM   = 0x01



import socket
import struct
import uuid

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader
from ptpip.packet.stream_writer import StreamWriter

class InitCmdReq(Packet):
    def __init__(
        self,
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None
    ):
        super(InitCmdReq, self).__init__(
            PacketType.InitCmdReq,
            data = data,
            transactionId = transactionId,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        if data is None:
            self.guid = uuid.uuid4().bytes
            self.hostname = socket.gethostname() + '\x00'
            self.hostname = self.hostname.encode()
        else:
            reader = StreamReader(data = data)
            self.guid = reader.readBytes(16)
            self.hostname = reader.readString()

    def pack(self):
        return StreamWriter() \
            .writeUint32(self.type.value) \
            .writeBytes(self.guid) \
            .writeBytes(self.hostname) \
            .data

    def __str__(self):
        return 'InitCmdReq: ' + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'guid: ' + str(self.guid) + "\n" \
            + "\t" + 'hostname: ' + str(self.hostname) + "\n"



from enum import Enum

class StillCaptureMode(Enum):
    Undefined               = 0x0000
    SingleFrameShooting     = 0x0001
    Burst                   = 0x0002 # ContinuousShootingH
    Timelapse               = 0x0003

    ContinuousShootingL     = 0x8010
    SelfTimer               = 0x8011 # @see BusrtNumber
    QuickResponseRemote     = 0x8014
    TwoSecondsDelayedRemote = 0x8015
    QuietShooting           = 0x8016



from enum import Enum

# Indicates “AE lock timer – Remote control standby time”
# in the custom menu.
class RemoteControlDelay(Enum):
    Min1 = 0x00
    Min5 = 0x01
    Min10 = 0x02
    Min15 = 0x03



from enum import Enum

class VendorExtension(Enum):
    Undefined               = 0x00000000
    EastmanKodak            = 0x00000001
    SeikoEpson              = 0x00000002
    Agilent                 = 0x00000003
    Polaroid                = 0x00000004
    AgfaGevaert             = 0x00000005
    Microsoft               = 0x00000006
    Equinox                 = 0x00000007
    Viewquest               = 0x00000008
    STMicroelectronics      = 0x00000009
    Nikon                   = 0x0000000A
    Canon                   = 0x0000000B
    FotoNation              = 0x0000000C
    PENTAX                  = 0x0000000D
    Fuji                    = 0x0000000E
    Sony                    = 0x00000011
    NDD                     = 0x00000012 # NDD Medical Technologies
    Samsung                 = 0x0000001A
    Parrot                  = 0x0000001B
    Panasonic               = 0x0000001C



import asyncio

from ptpip.constants.object_format import ObjectFormat
from ptpip.constants.response_code import ResponseCode

from ptpip.client import PtpIpClient

class GetNumObjectsCommand():
    def __init__(self, storageId, objectFormatId, objectFormatName, handle):
        super(GetNumObjectsCommand, self).__init__()

        self.storageId = storageId
        self.objectFormatId = objectFormatId
        self.objectFormatName = objectFormatName
        self.handle = handle

    async def run(self, client: PtpIpClient):

        objectFormat = self.objectFormatId

        if objectFormat is None \
            and self.objectFormatName in ObjectFormat._value2member_map_ \
        :
            objectFormat = ObjectFormat[self.objectFormatName].value \

        response = await client.getNumObjects(
            self.storageId,
            objectFormatId = objectFormat,
            handle = self.handle
        )

        if response is None:
            raise(Exception('Couldn\'t get the number of objects'))

        if isinstance(response, ResponseCode):
            raise(Exception('Error while fetching the number of objects: ' + response.name))

        print(str(response))



from enum import Enum

class ExposureProgramMode(Enum):
    Undefined                       = 0x0000
    Manual                          = 0x0001
    ProgramAuto                     = 0x0002
    AperturePriorityAuto            = 0x0003
    ShutterPriorityAuto             = 0x0004
    SceneModeAuto                   = 0x8010
    SceneModePortrait               = 0x8011
    SceneModeLandscape              = 0x8012
    SceneModeCloseUp                = 0x8013
    SceneModeSports                 = 0x8014
    SceneModeNightPortrait          = 0x8015
    SceneModeFlashProhibitionAuto   = 0x8016
    SceneModeChild 					= 0x8017
    SceneModeScene                  = 0x8018
    ColorSketch                     = 0x8019



import asyncio

from ptpip.data_object.storage_info import StorageInfo

from ptpip.client import PtpIpClient

class GetStorageInfoCommand():
    def __init__(self, storageId):
        super(GetStorageInfoCommand, self).__init__()

        self.storageId = storageId

    async def run(self, client: PtpIpClient):
        storage = await client.getStorageInfo(self.storageId)

        if not isinstance(storage, StorageInfo):
            raise(Exception('Error while fetching storage info: ' + str(storage)))

        print(str(storage))



from enum import Enum

class SceneMode(Enum):
    NightLandscape  = 0x00 # 0
    PartyIndoor     = 0x01 # 1
    BeachSnow       = 0x02 # 2
    Sunset          = 0x03 # 3
    DuskDawn        = 0x04 # 4
    PetPortrait     = 0x05 # 5
    Candlelight     = 0x06 # 6
    Blossom         = 0x07 # 7
    AutumnColors    = 0x08 # 8
    Food            = 0x09 # 9
    Silhouette      = 0x0A # 10
    HighKey         = 0x0B # 11
    LowKey          = 0x0C # 12



from enum import Enum

class ResponseCode(Enum):
    # PTP Response Code 0x2000..0x2FFF
    Undefined                                = 0x2000
    OK                                       = 0x2001
    GeneralError                             = 0x2002
    SessionNotOpen                           = 0x2003
    InvalidTransactionID                     = 0x2004
    OperationNotSupported                    = 0x2005
    ParameterNotSupported                    = 0x2006
    IncompleteTransfer                       = 0x2007
    InvalidStorageID                         = 0x2008
    InvalidObjectHandle                      = 0x2009
    DevicePropNotSupported                   = 0x200A
    InvalidObjectFormatCode                  = 0x200B
    StoreFull                                = 0x200C
    ObjectWriteProtected                     = 0x200D
    StoreReadOnly                            = 0x200E
    AccessDenied                             = 0x200F
    NoThumbnailPresent                       = 0x2010
    SelfTestFailed                           = 0x2011
    PartialDeletion                          = 0x2012
    StoreNotAvailable                        = 0x2013
    SpecificationByFormatUnsupported         = 0x2014
    NoValidObjectInfo                        = 0x2015
    InvalidCodeFormat                        = 0x2016
    UnknownVendorCode                        = 0x2017
    CaptureAlreadyTerminated                 = 0x2018
    DeviceBusy                               = 0x2019
    InvalidParentObject                      = 0x201A
    InvalidDevicePropFormat                  = 0x201B
    InvalidDevicePropValue                   = 0x201C
    InvalidParameter                         = 0x201D
    SessionAlreadyOpen                       = 0x201E
    TransactionCancelled                     = 0x201F
    SpecificationOfDestinationUnsupported    = 0x2020
    InvalidEnumHandle                        = 0x2021
    NoStreamEnabled                          = 0x2022
    InvalidDataset                           = 0x2023

    # Vendor Extension Response Code 0xA000..0xA7FF
    NIKON_HardwareError                      = 0xA001
    NIKON_OutOfFocus                         = 0xA002
    NIKON_ChangeCameraModeFailed             = 0xA003
    NIKON_InvalidStatus                      = 0xA004
    NIKON_SetPropertyNotSupport              = 0xA005
    NIKON_WbPresetError                      = 0xA006
    NIKON_DustReferenceError                 = 0xA007
    NIKON_ShutterSpeedBulb                   = 0xA008
    NIKON_MirrorUpSequence                   = 0xA009
    NIKON_CameraModeNotAdjustFnumber         = 0xA00A
    NIKON_NotLiveView                        = 0xA00B
    NIKON_MfDriveStepEnd                     = 0xA00C
    NIKON_MfDriveStepInsufficiency           = 0xA00E
    NIKON_AdvancedTransferCancel             = 0xA022

    NIKON_UnknownError                       = 0xA081 # ObjectFormatError ?!

    NIKON_Bulb_Release_Busy                  = 0xA200
    NIKON_Silent_Release_Busy                = 0xA201
    NIKON_MovieFrame_Release_Busy            = 0xA202
    NIKON_ShutterSpeed_Time                  = 0xA204
    NIKON_Waiting2ndRelease                  = 0xA207
    NIKON_MirrorUpCaptureAlreadyStart        = 0xA208
    NIKON_InvalidSBAttributeValue            = 0xA209

    # MTP Response Code 0xA800..0xAFFF
    MTP_Undefined                            = 0xA800
    MTP_InvalidObjectPropCode                = 0xA801
    MTP_InvalidObjectPropFormat              = 0xA802
    MTP_InvalidObjectPropValue               = 0xA803
    MTP_InvalidObjectReference               = 0xA804
    MTP_InvalidDataset                       = 0xA806
    MTP_SpecificationByGroupUnsupported      = 0xA807
    MTP_SpecificationByDepthUnsupported      = 0xA808
    MTP_ObjectTooLarge                       = 0xA809
    MTP_ObjectPropNotSupported               = 0xA80A

    MTP_InvalidMediaSessionID                = 0xA170
    MTP_MediaSessionLimitReached             = 0xA171
    MTP_NoMoreData                           = 0xA172

    MTP_InvalidWFCSyntax                     = 0xA121
    MTP_WFCVersionNotSupported               = 0xA122





from enum import Enum

class AssociationType(Enum):
    GenericFolder       = 0x0001
    Album               = 0x0002
    TimeSequence        = 0x0003
    HorizontalPanoramic = 0x0004
    VerticalPanoramic   = 0x0005
    Panoramic2D         = 0x0006
    AncillaryData       = 0x0007



from enum import Enum

class PropertyType(Enum):
    Undefined       = 0x0000
    Int8            = 0x0001
    Uint8           = 0x0002
    Int16           = 0x0003
    Uint16          = 0x0004
    Int32           = 0x0005
    Uint32          = 0x0006
    Int64           = 0x0007
    Uint64          = 0x0008
    Int128          = 0x0009
    Uint128         = 0x000A

    Int8Array       = 0x4001
    Uint8Array      = 0x4002
    Int16Array      = 0x4003
    Uint16Array     = 0x4004
    Int32Array      = 0x4005
    Uint32Array     = 0x4006
    Int64Array      = 0x4007
    Uint64Array     = 0x4008
    Int128Array     = 0x4009
    Uint128Array    = 0x400A

    String          = 0xFFFF



from ptpip.constants.storage.access_capability import AccessCapability
from ptpip.constants.storage.type import StorageType
from ptpip.constants.storage.filesystem_type import FilesystemType

from ptpip.packet.stream_reader import StreamReader

class StorageInfo():
	def __init__(self, packet, data):
		super(StorageInfo, self).__init__()

		self.packet = packet

		if data is not None:
			reader = StreamReader(data)
			self.typeId = reader.readUint16()
			self.type = StorageType(self.typeId) \
				if self.typeId in StorageType._value2member_map_ \
				else None

			self.filesystemTypeId = reader.readUint16()
			self.filesystemType = FilesystemType(self.filesystemTypeId) \
				if self.filesystemTypeId in FilesystemType._value2member_map_ \
				else None

			self.accessCapabilityId = reader.readUint16()
			self.accessCapability = AccessCapability(self.accessCapabilityId) \
				if self.accessCapabilityId in AccessCapability._value2member_map_ \
				else None

			self.maxCapacity = reader.readUint64()
			self.freeSpaceInBytes = reader.readUint64()
			self.freeSpaceInImages = reader.readUint32()
			self.description = reader.readUint8()
			self.volumeLabel = reader.readString()

	def sizeof(self, num, suffix = "B"):
		for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
			if abs(num) < 1024.0:
				return f"{num:3.1f}{unit}{suffix}"
			num /= 1024.0

		return f"{num:.1f}Yi{suffix}"

	def __str__(self):
		return 'StorageInfo: ' + "\n" \
			+ "\t" + 'typeId: ' + str(self.typeId) + "\n" \
			+ "\t" + 'type: ' + (
				self.type.name \
				if self.type is not None \
				else ''
			) + "\n" \
			+ "\t" + 'filesystemTypeId: ' + str(self.filesystemTypeId) + "\n" \
			+ "\t" + 'filesystemType: ' + (
				self.filesystemType.name \
				if self.filesystemType is not None \
				else ''
			) + "\n" \
			+ "\t" + 'accessCapabilityId: ' + str(self.accessCapabilityId) + "\n" \
			+ "\t" + 'accessCapability: ' + (
				self.accessCapability.name \
					if self.accessCapability is not None \
					else ''
			) + "\n" \
			+ "\t" + 'maxCapacity: ' + self.sizeof(self.maxCapacity) + "\n" \
			+ "\t" + 'freeSpaceInBytes: ' + self.sizeof(self.freeSpaceInBytes) + "\n" \
			+ "\t" + 'freeSpaceInImages: ' + str(self.freeSpaceInImages) + "\n" \
			+ "\t" + 'description: ' + str(self.description) + "\n" \
			+ "\t" + 'volumeLabel: ' + str(self.volumeLabel) + "\n"



from enum import Enum

# Indicates “Shooting, recording, and display – Sequential numbering mode”
# in the custom menu.
class NumberingMode(Enum):
    Off     = 0x00
    On      = 0x01
    Reset   = 0x02



from enum import Enum

# Indicates the choice of “Shooting, recording, and display
# - Date imprint setting –Birthday counter”
# in the custom menu.
class DateCounterSelect(Enum):
    First   = 0x00
    Second  = 0x01
    Third   = 0x02



from enum import Enum

class WhiteBalance(Enum):
	Undefined 		= 0x0000
	Auto 			= 0x0002
	Sunny 			= 0x0004
	Fluorescent 	= 0x0005
	Incandescent 	= 0x0006
	Flash 			= 0x0007

	Cloudy 			= 0x8010
	SunnyShade 		= 0x8011
	Preset 			= 0x8013



import asyncio
import struct

from threading import Thread

from ptpip.connection import Connection

from ptpip.constants.cmd_type import CmdType
from ptpip.constants.event_type import EventType
from ptpip.constants.property_type import PropertyType
from ptpip.constants.response_code import ResponseCode
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.constants.device.property_type import DevicePropertyType

from ptpip.packet.stream_reader import StreamReader
from ptpip.packet.stream_writer import StreamWriter
from ptpip.packet.cmd_request import CmdRequest
from ptpip.packet.cmd_response import CmdResponse

from ptpip.data_object.data_object import DataObject
from ptpip.data_object.device_info import DeviceInfo
from ptpip.data_object.device_prop_desc import DevicePropDesc
from ptpip.data_object.object_handle_array import ObjectHandleArray
from ptpip.data_object.storage_id_array import StorageIdArray
from ptpip.data_object.storage_info import StorageInfo

class PtpIpClient():
    def __init__(
        self,
        middleware,
        host = None,
        port = None,
        communicationThreadDelay = 0,
        debug = False
    ):
        super(PtpIpClient, self).__init__()

        self.host = host
        self.port = port
        self.communicationThreadDelay = communicationThreadDelay
        self.debug = debug == True

        self.conn = Connection(debug = debug)
        self.conn.open(
            host = self.host,
            port = self.port
        )

        # Start the Thread which is constantly checking the status of the camera
        # and which is processing new command packages which should be send
        threadComm = Thread(
            target = self.conn.communicationThread,
            args = (communicationThreadDelay,)
        )

        threadComm.daemon = True
        threadComm.start()

        eventLoop = asyncio.get_event_loop()

        try:
            eventLoop.run_until_complete(
                middleware(self)
            )
        except KeyboardInterrupt:
            eventLoop.stop()
            pass

    async def getDeviceInfo(self, delay = 0, transactionId = None):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetDeviceInfo.value
        )
        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId :
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, DeviceInfo):
                    return objData
                elif isinstance(objData, DataObject) \
                    and isinstance(objData.packet, CmdResponse) \
                :
                    return objData.packet.code
                else:
                    return None

    async def discoverDevicePropDesc(
        self,
        device: DeviceInfo,
        delay = 0,
        more = False
    ):
        discovered = []
        for propId in range(1, 65536):
            if propId in device.properties \
                or (
                    not more
                    and propId not in DevicePropertyType._value2member_map_
                ) \
            :
                continue

            response = await self.getDevicePropDesc(
                propId,
                delay = delay,
                transactionId = self.conn.createTransaction()
            )

            if isinstance(response, ResponseCode):
                continue

            discovered.append(response)

        return discovered


    async def getDevicePropDesc(self, prop, delay = 0, transactionId = None):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetDevicePropDesc.value,
            param1 = prop
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, DevicePropDesc):
                    return objData
                elif isinstance(objData, DataObject) \
                    and isinstance(objData.packet, CmdResponse) \
                :
                    return objData.packet.code
                else:
                    return None

    async def setDevicePropValue(
        self,
        prop,
        propType: PropertyType,
        value,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.SetDevicePropValue.value,
            param1 = prop,
            dataObjectTransferMode = DataObjectTransferMode.Send,
            dataObject = DataObject(
                data = StreamWriter() \
                    .writeType(propType.name, value) \
                    .data
            )
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, DataObject):
                    if isinstance(objData.packet, CmdResponse) \
                        and objData.packet.code \
                            != ResponseCode.OK.value \
                    :
                        return objData.packet.code
                    return objData
                else:
                    return None

    async def getDevicePropValue(
        self,
        prop,
        propType: PropertyType,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetDevicePropValue.value,
            param1 = prop
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, DataObject):
                    if isinstance(objData.packet, CmdResponse) \
                        and objData.packet.code \
                            != ResponseCode.OK.value \
                    :
                        return objData.packet.code
                    return StreamReader(objData.data) \
                        .readType(propType.name)
                else:
                    return None

    async def initCapture(
        self,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.InitiateCapture.value,
            param1 = 0,
            param2 = 0
        )

        self.conn.sendCmd(cmd)

        foundEvent = False
        async for event in self.conn.listenEventQueue(delay = delay):
            if event.type == EventType.ObjectAdded:
                self.conn.eventQueue.remove(event)
                foundEvent = event
                break

        if not foundEvent:
            raise(Exception('Couldn\'t receive capture event'))

        objTransactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = objTransactionId,
            cmd = CmdType.GetObject.value,
            param1 = event.parameter
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if isinstance(objData, DataObject) \
                and objData.packet.transactionId == objTransactionId \
            :
                self.conn.objectQueue.remove(objData)
                return objData.data

    async def getStorageIds(
        self,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetStorageIDs.value
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, StorageIdArray):
                    return objData
                elif isinstance(objData, CmdResponse):
                    return objData.code
                else:
                    return None

    async def getStorageInfo(
        self,
        storageId,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetStorageInfo.value,
            param1 = storageId
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, StorageInfo):
                    return objData
                elif isinstance(objData, CmdResponse):
                    return objData.code
                else:
                    return None

    async def getNumObjects(
        self,
        storageId,
        objectFormatId = None,
        handle = None,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetNumObjects.value,
            param1 = storageId,
            param2 = objectFormatId,
            param3 = handle
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData.packet, CmdResponse):
                    if objData.packet.code != ResponseCode.OK:
                        return objData.packet.code
                    elif len(objData.packet.parameters) > 0:
                        return objData.packet.parameters[0];
                    else:
                        return None
                else:
                    return None

    async def getObjectHandles(
        self,
        storageId = None,
        objectFormatId = None,
        handle = None,
        allStorages = True,
        allFormats = True,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            transactionId = self.conn.createTransaction()

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetObjectHandles.value,
            param1 = storageId \
                if storageId is not None \
                    or not allStorages \
                else 0xFFFFFFFF,
            param2 = objectFormatId \
                if objectFormatId is not None \
                    or not allFormats \
                else 0xFFFFFFFF,
            param3 = handle
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if objData.packet.transactionId == transactionId:
                self.conn.objectQueue.remove(objData)
                if isinstance(objData, ObjectHandleArray):
                    return objData
                elif isinstance(objData, CmdResponse):
                    return objData.code
                else:
                    return None

    async def getPictureControlCapabilities(
        self,
        picCtrlItem,
        defaultFlag = 0x00,
        delay = 0,
        transactionId = None
    ):
        if transactionId == None:
            self.lastTransactionId += 1
            transactionId = self.lastTransactionId

        cmd = CmdRequest(
            transactionId = transactionId,
            cmd = CmdType.GetPicCtrlCapability.value,
            param1 = picCtrlItem,
            param2 = defaultFlag
        )

        self.conn.sendCmd(cmd)

        async for objData in self.conn.listenObjectDataQueue(delay = delay):
            if isinstance(objData, DataObject) \
                and objData.packet.transactionId == transactionId \
            :
                self.objectQueue.remove(objData)

                return objData



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader
from ptpip.packet.stream_writer import StreamWriter

class EndDataPacket(Packet):
    def __init__(
        self,
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        request: Packet = None
    ):
        super(EndDataPacket, self).__init__(
            PacketType.EndData,
            data = data,
            transactionId = transactionId,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        self.request = request

        if data is not None:
            reader = StreamReader(data = data)
            self.transactionId = reader.readUint32()
            self.data = reader.readRest()
        else:
            self.transactionId = transactionId

    def pack(self):
        return StreamWriter() \
            .writeUint32(self.type.value) \
            .writeUint32(self.transactionId) \
            .data

    def __str__(self):
        return 'EndDataPacket: ' + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'transactionId: ' + str(self.transactionId) + "\n" \
            + "\t" + 'data: ' + str(self.data) + "\n"



from ptpip.packet.stream_reader import StreamReader

class Uint32Array():
    def __init__(self, packet, data):
        super(Uint32Array, self).__init__()

        self.packet = packet
        self.length = 0
        self.elements = []

        if data is not None:
            reader = StreamReader(data)
            self.length = reader.readUint32()

            for elt in range(0, self.length):
                self.elements.append(reader.readUint32())

    def __str__(self):
        return self.__class__.__name__ + ':' + "\n" \
            + "\t" + 'length: ' + str(self.length) + "\n" \
            + "\t" + 'elements: ' + str(self.elements) + "\n"



from ptpip.constants.cmd_type import CmdType
from ptpip.constants.event_type import EventType
from ptpip.constants.object_format import ObjectFormat

from ptpip.constants.device.property_type import DevicePropertyType
from ptpip.constants.device.functional_mode import FunctionalMode
from ptpip.constants.device.vendor_extension import VendorExtension

from ptpip.packet.stream_reader import StreamReader
from ptpip.data_object.data_object import DataObject

class DeviceInfo():
    def __init__(self, packet, data):
        super(DeviceInfo, self).__init__()
        self.packet = packet

        reader = StreamReader(data = data)
        self.standardVersion        = reader.readUint16()
        self.vendorExtensionId      = reader.readUint32()
        self.vendorExtensionVersion = reader.readUint16()
        self.vendorExtensionDesc    = reader.readString()
        self.functionalMode         = reader.readUint16()
        self.operations             = reader.readUint16Array()
        self.events                 = reader.readUint16Array()
        self.properties             = reader.readUint16Array()
        self.captureFormats         = reader.readUint16Array()
        self.imageFormats           = reader.readUint16Array()
        self.manufacturer           = reader.readString()
        self.model                  = reader.readString()
        self.deviceVersion          = reader.readString()
        self.serialNumber           = reader.readString()

    def vendorExtensionIdStr(self):
        return VendorExtension(self.vendorExtensionId).name \
            if self.vendorExtensionId in VendorExtension._value2member_map_ \
            else str(self.vendorExtensionId)

    def functionalModeStr(self):
        return FunctionalMode(self.functionalMode).name \
            if self.functionalMode in FunctionalMode._value2member_map_ \
            else str(self.functionalMode)

    def operationStr(self, operation):
        return CmdType(operation).name \
            if operation in CmdType._value2member_map_ \
            else str(operation)

    def eventStr(self, event):
        return EventType(event).name \
            if event in EventType._value2member_map_ \
            else str(event)

    def objectFormatStr(self, objectFormat):
        return ObjectFormat(objectFormat).name \
            if objectFormat in ObjectFormat._value2member_map_ \
            else str(objectFormat)

    def __str__(self):
        sOps = "[\n"
        for i in range(0, len(self.operations)):
            sOps += "\t" + self.operationStr(self.operations[i]) + ",\n"
        sOps += "]"

        sEvents = "[\n"
        for i in range(0, len(self.events)):
            sEvents += "\t" + self.eventStr(self.events[i]) + ",\n"
        sEvents += "]"

        sProps = "[\n"
        for i in range(0, len(self.properties)):
            sProp = DevicePropertyType(self.properties[i]).name \
                if self.properties[i] in DevicePropertyType._value2member_map_ \
                else str(self.properties[i])

            sProps += "\t" + sProp + ",\n"
        sProps += "]"

        sCaptureFormats = "[\n"
        for i in range(0, len(self.captureFormats)):
            sCaptureFormats += "\t" + self.objectFormatStr(self.captureFormats[i]) + ",\n"
        sCaptureFormats += "]"

        sImageFormats = "[\n"
        for i in range(0, len(self.imageFormats)):
            sImageFormats += "\t" + self.objectFormatStr(self.imageFormats[i]) + ",\n"
        sImageFormats += "]"

        return '---- DeviceInfo ----' + "\n" \
            + 'standardVersion: ' + str(self.standardVersion) + "\n" \
            + 'vendorExtensionId: ' + self.vendorExtensionIdStr() + "\n" \
            + 'vendorExtensionVersion: ' + str(self.vendorExtensionVersion) + "\n" \
            + 'vendorExtensionDesc: ' + str(self.vendorExtensionDesc) + "\n" \
            + 'functionalMode: ' + self.functionalModeStr() + "\n" \
            + 'operationsSupported: ' + sOps + "\n" \
            + 'eventsSupported: ' + sEvents + "\n" \
            + 'devicePropertiesSupported: ' + sProps + "\n" \
            + 'captureFormats: ' + sCaptureFormats + "\n" \
            + 'imageFormats: ' + sImageFormats + "\n" \
            + 'manufacturer: ' + str(self.manufacturer) + "\n" \
            + 'model: ' + str(self.model) + "\n" \
            + 'deviceVersion: ' + str(self.deviceVersion) + "\n" \
            + 'serialNumber: ' + str(self.serialNumber) + "\n" \
            + '---- End DeviceInfo ----'



from enum import Enum

class LensType(Enum):
    Undefined                                   = 0
    DTypeLenWithDistanceEncoder                 = 1
    VRLensWithAntiVibrationMechanism            = 2
    DXLensExclusiveUseOfNikon                   = 4
    AFSLens                                     = 8
    LensSupportingAutomaticDistortionCorrection = 16
    Unknown1                                    = 32
    Unknown2                                    = 64



from enum import Enum
class CmdType(Enum):
    # PTP Operation Code 0x1000..0x1FFF
    Undefined                   = 0x1000
    GetDeviceInfo               = 0x1001
    OpenSession                 = 0x1002
    CloseSession                = 0x1003
    GetStorageIDs               = 0x1004
    GetStorageInfo              = 0x1005
    GetNumObjects               = 0x1006
    GetObjectHandles            = 0x1007
    GetObjectInfo               = 0x1008
    GetObject                   = 0x1009
    GetThumb                    = 0x100A
    DeleteObject                = 0x100B
    SendObjectInfo              = 0x100C
    SendObject                  = 0x100D
    InitiateCapture             = 0x100E
    FormatStore                 = 0x100F
    ResetDevice                 = 0x1010
    SelfTest                    = 0x1011
    SetObjectProtection         = 0x1012
    PowerDown                   = 0x1013
    GetDevicePropDesc           = 0x1014
    GetDevicePropValue          = 0x1015
    SetDevicePropValue          = 0x1016
    ResetDevicePropValue        = 0x1017
    TerminateOpenCapture        = 0x1018
    MoveObject                  = 0x1019
    CopyObject                  = 0x101A
    GetPartialObject            = 0x101B
    InitiateOpenCapture         = 0x101C
    StartEnumHandles            = 0x101D
    EnumHandles                 = 0x101E
    StopEnumHandles             = 0x101F
    GetVendorExtensionMapss     = 0x1020
    GetVendorDeviceInfo         = 0x1021
    GetResizedImageObject       = 0x1022
    GetFilesystemManifest       = 0x1023
    GetStreamInfo               = 0x1024
    GetStream                   = 0x1025

    # Vendor Extension Operation Code 0x9000..0x97FF

    GetProfileAllData           = 0x9006
    SendProfileData             = 0x9007
    DeleteProfile               = 0x9008
    SetProfileData              = 0x9009
    AdvancedTransfer            = 0x9010
    GetFileInfoInBlock          = 0x9011

    InitiateCaptureRecInSdram   = 0x90C0
    AfDrive                     = 0x90C1
    ChangeCameraMode            = 0x90C2
    DeleteImagesInSdram         = 0x90C3
    GetLargeThumb               = 0x90C4
    GetEvent                    = 0x90C7
    DeviceReady                 = 0x90C8 # 37064
    SetPreWbData                = 0x90C9
    GetVendorPropCodes          = 0x90CA
    AfAndCaptureRecInSdram      = 0x90CB
    GetPicCtrlData              = 0x90CC
    SetPicCtrlData              = 0x90CD
    DeleteCustomPicCtrl         = 0x90CE
    GetPicCtrlCapability        = 0x90CF
    GetDevicePTPIPInfo          = 0x90E0

    GetPreviewImg               = 0x9200
    StartLiveView               = 0x9201
    EndLiveView                 = 0x9202
    GetLiveViewImage            = 0x9203
    MfDrive                     = 0x9204
    ChangeAfArea                = 0x9205
    AfDriveCancel               = 0x9206
    InitiateCaptureRecInMedia   = 0x9207
    GetVendorStorageIDs         = 0x9209
    StartMovieRecInCard         = 0x920A
    EndMovieRec                 = 0x920B
    TerminateCapture            = 0x920C
    GetFhdPicture               = 0x920F

    GetPartialObjectHighSpeed   = 0x9400
    StartSpotWb                 = 0x9402
    EndSpotWb                   = 0x9403
    ChangeSpotWbArea            = 0x9404
    MeasureSpotWb               = 0x9405
    EndSpotWbResultDisp         = 0x9406
    SetTransferListLock         = 0x9407
    GetTransferList             = 0x9408
    NotifyFileAcquisitionStart  = 0x9409
    NotifyFileAcquisitionEnd    = 0x940A
    GetSpecificSizeObject       = 0x940B
    CancelImagesInSDRAM         = 0x940c
    GetSBHandles                = 0x9414
    GetSBAttrDesc               = 0x9415
    GetSBAttrValue              = 0x9416
    SetSBAttrValue              = 0x9417
    GetSBGroupAttrDesc          = 0x9418
    GetSBGroupAttrValue         = 0x9419
    SetSBGroupAttrValue         = 0x941a
    TestFlash                   = 0x941b
    GetEventEx                  = 0x941c
    MirrorUpCancel              = 0x941d
    PowerZoomByFocalLength      = 0x941e
    ActiveSelectionControl      = 0x941f
    SaveCameraSetting           = 0x9420
    GetObjectSize               = 0x9421
    ChangeMonitorOff            = 0x9422
    GetLiveViewCompressedSize   = 0x9423
    StartTracking               = 0x9424
    EndTracking                 = 0x9425
    ChangeAELock                = 0x9426
    GetLiveViewImageEx          = 0x9428
    GetPartialObjectEx          = 0x9431
    GetManualSettingLensData    = 0x9432
    InitiatePixelMapping        = 0x9433
    GetObjectsMetaData          = 0x9434
    ChangeApplicationMode       = 0x9435
    ResetMenu                   = 0x9436

    GetDevicePropEx             = 0x9504

    # MTP Operation Code 0x9800..0x9FFF
    GetObjectPropsSupported     = 0x9801
    GetObjectPropDesc           = 0x9802
    GetObjectPropValue          = 0x9803
    SetObjectPropValue          = 0x9804
    GetObjectPropList           = 0x9805
    SetObjectPropList           = 0x9806
    GetInterdependendPropdesc   = 0x9807
    SendObjectPropList          = 0x9808
    GetObjectReferences         = 0x9810
    SetObjectReferences         = 0x9811
    UpdateDeviceFirmware        = 0x9812
    Skip                        = 0x9820

    WlanPowerControl            = 0x99A1



from enum import Enum

# Indicates the picture control item whose setting is currently valid.
class ActivePicCtrlItem(Enum):
    Standard    = 0x01
    Neutral     = 0x02
    Vivid       = 0x03
    Monochrome  = 0x04
    Portrait    = 0x05
    Landscape   = 0x06

    Option1     = 0x65
    Option2     = 0x66
    Option3     = 0x67
    Option4     = 0x68

    Custom1     = 0xC9
    Custom2     = 0xCA
    Custom3     = 0xCB
    Custom4     = 0xCC
    Custom5     = 0xCD
    Custom6     = 0xCE
    Custom7     = 0xCF
    Custom8     = 0xD0
    Custom9     = 0xD1



from enum import Enum

# Indicates the time that is set in “Auto meter-off delay”
# of “AE lock timer – Power-offtime” in the custom menu
class AutoMeterOffDelay(Enum):
    Sec4    = 0x00
    Sec8    = 0x01
    Sec20   = 0x02
    Min1    = 0x03
    Min30   = 0x04



import asyncio

from ptpip.constants.device.property_type import DevicePropertyType

from ptpip.client import PtpIpClient

class GetDevicePropDescCommand():
    def __init__(self, name, id):
        super(GetDevicePropDescCommand, self).__init__()

        self.name = name
        self.id = id

    async def run(self, client: PtpIpClient):
        prop = self.id if self.id != None else DevicePropertyType[self.name].value

        propDesc = await client.getDevicePropDesc(
            prop = prop
        )

        print(str(propDesc))



from enum import Enum

# Indicates “AE lock timer – Power-off time” in the custom menu.
class AutoOffTime(Enum):
    Short       = 0x00
    Normal      = 0x01
    Long        = 0x02
    Customize   = 0x03



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.response_code import ResponseCode
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

from ptpip.packet.packet import Packet
from ptpip.packet.stream_reader import StreamReader

class CmdResponse(Packet):
    def __init__(
        self,
        data = None,
        transactionId = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None,
        request: Packet = None,
        code = None
    ):
        super(CmdResponse, self).__init__(
            PacketType.CmdResponse,
            data = data,
            transactionId = transactionId,
            dataObject = dataObject,
            dataObjectTransferMode = dataObjectTransferMode
        )

        self.request = request
        self.code = None
        self.transactionId = None
        self.parameters = []

        if data is not None:
            reader = StreamReader(data = data)
            code = reader.readUint16()
            self.code = ResponseCode(code) \
                if code in ResponseCode._value2member_map_ \
                else code
            self.transactionId = reader.readUint32()

            while reader.pos + 4 <= len(reader.data):
                self.parameters.append(reader.readUint32())

    def __str__(self):
        return 'CmdResponse: ' + "\n" \
            + "\t" + 'type: ' + str(self.type) + "\n" \
            + "\t" + 'code: ' + str(self.code) + "\n" \
            + "\t" + 'transactionId: ' + str(self.transactionId) + "\n" \
            + "\t" + 'parameters: ' + str(self.parameters) + "\n"



import struct

class DataObject():
    def __init__(
        self,
        packet = None,
        data = None
    ):
        super(DataObject, self).__init__()

        self.packet = packet
        self.data = data

    def __str__(self):
        return 'DataObject: ' + "\n" \
            + "\t" + 'Packet: ' + str(self.packet) + "\n" \
            + "\t" + 'Data: ' + str(self.data) + "\n"



from enum import Enum

# Indicates “AE lock timer – Self-timer – The number of captured frames”
# in the custom menu.
class SelfTimerDelay(Enum):
    Sec2 = 0x00
    Sec5 = 0x01
    Sec10 = 0x02
    Sec20 = 0x03



import struct

from ptpip.event.event import Event

from ptpip.packet.stream_reader import StreamReader

class EventFactory():
    """
    This is a factory to produce an array of Event objects if it got passd a data reply
    from a GetEvent request 0x90C7
    """
    def __init__(self, data):
        super(EventFactory, self).__init__()
        self.events = []

        reader = StreamReader(data)

        nbEvents = reader.readUint16()

        counter = 1
        for idx in range(0, nbEvents):

            self.events.append(Event(
                reader.readUint16(),
                reader.readUint32()
            ))

    def getEvents(self):
        return self.events



from enum import Enum

class InternalFlashCompensation(Enum):
    Minus18SixthEV = -18
    Minus17SixthEV = -17
    Minus16SixthEV = -16
    Minus15SixthEV = -15
    Minus14SixthEV = -14
    Minus13SixthEV = -13
    Minus12SixthEV = -12
    Minus11SixthEV = -11
    Minus10SixthEV = -10
    Minus9SixthEV  = -9
    Minus8SixthEV  = -8
    Minus7SixthEV  = -7
    Minus6SixthEV  = -6
    Minus5SixthEV  = -5
    Minus4SixthEV  = -4
    Minus3SixthEV  = -3
    Minus2SixthEV  = -2
    Minus1SixthEV  = -1
    ZeroEV         = 0
    Plus1SixthEV   = 1
    Plus2SixthEV   = 2
    Plus3SixthEV   = 3
    Plus4SixthEV   = 4
    Plus5SixthEV   = 5
    Plus6SixthEV   = 6



from enum import Enum

class FocusMeteringMode(Enum):
    Undefined           = 0x0000
    DynamicAFMode       = 0x0002

    SinglePointAFMode   = 0x8010
    AutoAreaAFMode      = 0x8011
    Tracking3D11Points  = 0x8012



from enum import Enum

class FlashCompensation(Enum):
    Minus18SixthEV = -18
    Minus17SixthEV = -17
    Minus16SixthEV = -16
    Minus15SixthEV = -15
    Minus14SixthEV = -14
    Minus13SixthEV = -13
    Minus12SixthEV = -12
    Minus11SixthEV = -11
    Minus10SixthEV = -10
    Minus9SixthEV  = -9
    Minus8SixthEV  = -8
    Minus7SixthEV  = -7
    Minus6SixthEV  = -6
    Minus5SixthEV  = -5
    Minus4SixthEV  = -4
    Minus3SixthEV  = -3
    Minus2SixthEV  = -2
    Minus1SixthEV  = -1
    ZeroEV         = 0
    Plus1SixthEV   = 1
    Plus2SixthEV   = 2
    Plus3SixthEV   = 3
    Plus4SixthEV   = 4
    Plus5SixthEV   = 5
    Plus6SixthEV   = 6
    Plus7SixthEV   = 7
    Plus8SixthEV   = 8
    Plus9SixthEV   = 9
    Plus10SixthEV  = 10
    Plus11SixthEV  = 11
    Plus12SixthEV  = 12
    Plus13SixthEV  = 13
    Plus14SixthEV  = 14
    Plus15SixthEV  = 15
    Plus16SixthEV  = 16
    Plus17SixthEV  = 17
    Plus18SixthEV  = 18



import struct

from ptpip.constants.packet_type import PacketType
from ptpip.constants.data_object_transfer_mode import DataObjectTransferMode

from ptpip.data_object.data_object import DataObject

class Packet(object):
    def __init__(
        self,
        type: PacketType,
        transactionId = None,
        data = None,
        dataObject: DataObject = None,
        dataObjectTransferMode: DataObjectTransferMode = None
    ):
        super(Packet, self).__init__()

        self.type = type
        self.transactionId = transactionId
        self.data = data
        self.dataObject = dataObject
        self.dataObjectTransferMode = dataObjectTransferMode \
            if dataObjectTransferMode != None \
            else DataObjectTransferMode.Receive

    def pack(self):
        pass



from enum import Enum

class ExposureIndicateStatus(Enum):
    Minus60SixthEV  = -60
    Minus59SixthEV  = -59
    Minus58SixthEV  = -58
    Minus57SixthEV  = -57
    Minus56SixthEV  = -56
    Minus55SixthEV  = -55
    Minus54SixthEV  = -54
    Minus53SixthEV  = -53
    Minus52SixthEV  = -52
    Minus51SixthEV  = -51
    Minus50SixthEV  = -50
    Minus49SixthEV  = -49
    Minus48SixthEV  = -48
    Minus47SixthEV  = -47
    Minus46SixthEV  = -46
    Minus45SixthEV  = -45
    Minus44SixthEV  = -44
    Minus43SixthEV  = -43
    Minus42SixthEV  = -42
    Minus41SixthEV  = -41
    Minus40SixthEV  = -40
    Minus39SixthEV  = -39
    Minus38SixthEV  = -38
    Minus37SixthEV  = -37
    Minus36SixthEV  = -36
    Minus35SixthEV  = -35
    Minus34SixthEV  = -34
    Minus33SixthEV  = -33
    Minus32SixthEV  = -32
    Minus31SixthEV  = -31
    Minus30SixthEV  = -30
    Minus29SixthEV  = -29
    Minus28SixthEV  = -28
    Minus27SixthEV  = -27
    Minus26SixthEV  = -26
    Minus25SixthEV  = -25
    Minus24SixthEV  = -24
    Minus23SixthEV  = -23
    Minus22SixthEV  = -22
    Minus21SixthEV  = -21
    Minus20SixthEV  = -20
    Minus19SixthEV  = -19
    Minus18SixthEV  = -18
    Minus17SixthEV  = -17
    Minus16SixthEV  = -16
    Minus15SixthEV  = -15
    Minus14SixthEV  = -14
    Minus13SixthEV  = -13
    Minus12SixthEV  = -12
    Minus11SixthEV  = -11
    Minus10SixthEV  = -10
    Minus9SixthEV   = -9
    Minus8SixthEV   = -8
    Minus7SixthEV   = -7
    Minus6SixthEV   = -6
    Minus5SixthEV   = -5
    Minus4SixthEV   = -4
    Minus3SixthEV   = -3
    Minus2SixthEV   = -2
    Minus1SixthEV   = -1
    ZeroEV          = 0
    Plus1SixthEV    = 1
    Plus2SixthEV    = 2
    Plus3SixthEV    = 3
    Plus4SixthEV    = 4
    Plus5SixthEV    = 5
    Plus6SixthEV    = 6
    Plus7SixthEV    = 7
    Plus8SixthEV    = 8
    Plus9SixthEV    = 9
    Plus10SixthEV   = 10
    Plus11SixthEV   = 11
    Plus12SixthEV   = 12
    Plus13SixthEV   = 13
    Plus14SixthEV   = 14
    Plus15SixthEV   = 15
    Plus16SixthEV   = 16
    Plus17SixthEV   = 17
    Plus18SixthEV   = 18
    Plus19SixthEV   = 19
    Plus20SixthEV   = 20
    Plus21SixthEV   = 21
    Plus22SixthEV   = 22
    Plus23SixthEV   = 23
    Plus24SixthEV   = 24
    Plus25SixthEV   = 25
    Plus26SixthEV   = 26
    Plus27SixthEV   = 27
    Plus28SixthEV   = 28
    Plus29SixthEV   = 29
    Plus30SixthEV   = 30
    Plus31SixthEV   = 31
    Plus32SixthEV   = 32
    Plus33SixthEV   = 33
    Plus34SixthEV   = 34
    Plus35SixthEV   = 35
    Plus36SixthEV   = 36
    Plus37SixthEV   = 37
    Plus38SixthEV   = 38
    Plus39SixthEV   = 39
    Plus40SixthEV   = 40
    Plus41SixthEV   = 41
    Plus42SixthEV   = 42
    Plus43SixthEV   = 43
    Plus44SixthEV   = 44
    Plus45SixthEV   = 45
    Plus46SixthEV   = 46
    Plus47SixthEV   = 47
    Plus48SixthEV   = 48
    Plus49SixthEV   = 49
    Plus50SixthEV   = 50
    Plus51SixthEV   = 51
    Plus52SixthEV   = 52
    Plus53SixthEV   = 53
    Plus54SixthEV   = 54
    Plus55SixthEV   = 55
    Plus56SixthEV   = 56
    Plus57SixthEV   = 57
    Plus58SixthEV   = 58
    Plus59SixthEV   = 59
    Plus60SixthEV   = 60
